@@ -23,6 +23,8 @@
 #include "wine/unixlib.h"
 
 BOOL CNG_ImportPubKey(CERT_PUBLIC_KEY_INFO *pubKeyInfo, BCRYPT_KEY_HANDLE *key) DECLSPEC_HIDDEN;
+BOOL cng_prepare_signature(const char *alg_oid, BYTE *encoded_sig, DWORD encoded_sig_len,
+    BYTE **sig_value, DWORD *sig_len) DECLSPEC_HIDDEN;
 
 /* a few asn.1 tags we need */
 #define ASN_BOOL            (ASN_UNIVERSAL | ASN_PRIMITIVE | 0x01)
@@ -46,7 +48,7 @@ BOOL CNG_ImportPubKey(CERT_PUBLIC_KEY_INFO *pubKeyInfo, BCRYPT_KEY_HANDLE *key)
 /* Copies `len` bytes from `src` to `dst`,
  * reversing the order of the bytes
  */
-void CRYPT_CopyReversed(BYTE *dst, const BYTE *src, size_t len);
+void CRYPT_CopyReversed(BYTE *dst, const BYTE *src, size_t len) DECLSPEC_HIDDEN;
 
 BOOL CRYPT_EncodeLen(DWORD len, BYTE *pbEncoded, DWORD *pcbEncoded) DECLSPEC_HIDDEN;
 
@@ -349,6 +351,8 @@ WINECRYPT_CERTSTORE *CRYPT_FileNameOpenStoreW(HCRYPTPROV hCryptProv,
 void CRYPT_ImportSystemRootCertsToReg(void) DECLSPEC_HIDDEN;
 BOOL CRYPT_SerializeContextsToReg(HKEY key, DWORD flags, const WINE_CONTEXT_INTERFACE *contextInterface,
     HCERTSTORE memStore) DECLSPEC_HIDDEN;
+void CRYPT_RegReadSerializedFromReg(HKEY key, DWORD contextType,
+    HCERTSTORE store, DWORD disposition) DECLSPEC_HIDDEN;
 
 DWORD CRYPT_IsCertificateSelfSigned(const CERT_CONTEXT *cert) DECLSPEC_HIDDEN;
 
@@ -506,10 +510,9 @@ enum unix_funcs
     unix_import_store_cert,
     unix_close_cert_store,
     unix_enum_root_certs,
+    unix_funcs_count,
 };
 
-extern unixlib_handle_t crypt32_handle;
-
-#define CRYPT32_CALL( func, params ) __wine_unix_call( crypt32_handle, unix_ ## func, params )
+#define CRYPT32_CALL( func, params ) WINE_UNIX_CALL( unix_ ## func, params )
 
 #endif
