@@ -25,8 +25,6 @@
 
 #include "d3dcompiler_private.h"
 
-#include <vkd3d_shader.h>
-
 WINE_DEFAULT_DEBUG_CHANNEL(d3dcompiler);
 
 static HRESULT hresult_from_vkd3d_result(int vkd3d_result)
@@ -408,7 +406,7 @@ HRESULT WINAPI D3DCompile2(const void *data, SIZE_T data_size, const char *filen
     struct d3dcompiler_include_from_file include_from_file;
     struct vkd3d_shader_preprocess_info preprocess_info;
     struct vkd3d_shader_hlsl_source_info hlsl_info;
-    struct vkd3d_shader_compile_option options[2];
+    struct vkd3d_shader_compile_option options[3];
     struct vkd3d_shader_compile_info compile_info;
     struct vkd3d_shader_compile_option *option;
     struct vkd3d_shader_code byte_code;
@@ -455,13 +453,15 @@ HRESULT WINAPI D3DCompile2(const void *data, SIZE_T data_size, const char *filen
         include = &include_from_file.ID3DInclude_iface;
     }
 
-    if (flags & ~D3DCOMPILE_DEBUG)
+    if (flags & ~(D3DCOMPILE_DEBUG | D3DCOMPILE_PACK_MATRIX_ROW_MAJOR | D3DCOMPILE_PACK_MATRIX_COLUMN_MAJOR))
         FIXME("Ignoring flags %#x.\n", flags);
     if (effect_flags)
         FIXME("Ignoring effect flags %#x.\n", effect_flags);
     if (secondary_flags)
         FIXME("Ignoring secondary flags %#x.\n", secondary_flags);
 
+    if (shader_blob)
+        *shader_blob = NULL;
     if (messages_blob)
         *messages_blob = NULL;
 
@@ -519,6 +519,19 @@ HRESULT WINAPI D3DCompile2(const void *data, SIZE_T data_size, const char *filen
         option->value = true;
     }
 
+    if (flags & D3DCOMPILE_PACK_MATRIX_ROW_MAJOR)
+    {
+        option = &options[compile_info.option_count++];
+        option->name = VKD3D_SHADER_COMPILE_OPTION_PACK_MATRIX_ORDER;
+        option->value = VKD3D_SHADER_COMPILE_OPTION_PACK_MATRIX_ROW_MAJOR;
+    }
+    else if (flags & D3DCOMPILE_PACK_MATRIX_COLUMN_MAJOR)
+    {
+        option = &options[compile_info.option_count++];
+        option->name = VKD3D_SHADER_COMPILE_OPTION_PACK_MATRIX_ORDER;
+        option->value = VKD3D_SHADER_COMPILE_OPTION_PACK_MATRIX_COLUMN_MAJOR;
+    }
+
     ret = vkd3d_shader_compile(&compile_info, &byte_code, &messages);
 
     if (ret)
@@ -550,21 +563,25 @@ HRESULT WINAPI D3DCompile2(const void *data, SIZE_T data_size, const char *filen
             }
             memcpy(ID3D10Blob_GetBufferPointer(*messages_blob), messages, size);
         }
-        else
-            vkd3d_shader_free_messages(messages);
+
+        vkd3d_shader_free_messages(messages);
     }
 
-    if (!ret)
+    if (ret)
+        return hresult_from_vkd3d_result(ret);
+
+    if (!shader_blob)
     {
-        if (FAILED(hr = D3DCreateBlob(byte_code.size, shader_blob)))
-        {
-            vkd3d_shader_free_shader_code(&byte_code);
-            return hr;
-        }
-        memcpy(ID3D10Blob_GetBufferPointer(*shader_blob), byte_code.code, byte_code.size);
+        vkd3d_shader_free_shader_code(&byte_code);
+        return S_OK;
     }
 
-    return hresult_from_vkd3d_result(ret);
+    if (SUCCEEDED(hr = D3DCreateBlob(byte_code.size, shader_blob)))
+        memcpy(ID3D10Blob_GetBufferPointer(*shader_blob), byte_code.code, byte_code.size);
+
+    vkd3d_shader_free_shader_code(&byte_code);
+
+    return hr;
 }
 
 HRESULT WINAPI D3DCompile(const void *data, SIZE_T data_size, const char *filename,
@@ -596,11 +613,79 @@ HRESULT WINAPI D3DPreprocess(const void *data, SIZE_T size, const char *filename
     return preprocess_shader(data, size, filename, defines, include, shader, error_messages);
 }
 
-HRESULT WINAPI D3DDisassemble(const void *data, SIZE_T size, UINT flags, const char *comments, ID3DBlob **disassembly)
+HRESULT WINAPI D3DDisassemble(const void *data, SIZE_T size, UINT flags, const char *comments,
+        ID3DBlob **disassembly)
 {
-    FIXME("data %p, size %Iu, flags %#x, comments %p, disassembly %p stub!\n",
+    struct vkd3d_shader_compile_info compile_info;
+    enum vkd3d_shader_source_type source_type;
+    struct vkd3d_shader_code asm_code;
+    const char *ptr = data;
+    char *messages;
+    HRESULT hr;
+    int ret;
+
+    TRACE("data %p, size %Iu, flags %#x, comments %p, disassembly %p.\n",
             data, size, flags, comments, disassembly);
-    return E_NOTIMPL;
+
+    if (flags)
+        FIXME("Ignoring flags %#x.\n", flags);
+
+    if (comments)
+        FIXME("Ignoring comments %s.\n", debugstr_a(comments));
+
+#if D3D_COMPILER_VERSION >= 46
+    if (!size)
+        return E_INVALIDARG;
+#endif
+
+    if (size >= 4 && read_u32(&ptr) == TAG_DXBC)
+        source_type = VKD3D_SHADER_SOURCE_DXBC_TPF;
+    else
+        source_type = VKD3D_SHADER_SOURCE_D3D_BYTECODE;
+
+    compile_info.type = VKD3D_SHADER_STRUCTURE_TYPE_COMPILE_INFO;
+    compile_info.next = NULL;
+    compile_info.source.code = data;
+    compile_info.source.size = size;
+    compile_info.source_type = source_type;
+    compile_info.target_type = VKD3D_SHADER_TARGET_D3D_ASM;
+    compile_info.options = NULL;
+    compile_info.option_count = 0;
+    compile_info.log_level = VKD3D_SHADER_LOG_INFO;
+    compile_info.source_name = NULL;
+
+    ret = vkd3d_shader_compile(&compile_info, &asm_code, &messages);
+
+    if (ret)
+        ERR("Failed to disassemble shader, vkd3d result %d.\n", ret);
+
+    if (messages)
+    {
+        if (*messages && ERR_ON(d3dcompiler))
+        {
+            const char *ptr = messages;
+            const char *line;
+
+            ERR("Shader log:\n");
+            while ((line = get_line(&ptr)))
+            {
+                ERR("    %.*s", (int)(ptr - line), line);
+            }
+            ERR("\n");
+        }
+
+        vkd3d_shader_free_messages(messages);
+    }
+
+    if (ret)
+        return hresult_from_vkd3d_result(ret);
+
+    if (SUCCEEDED(hr = D3DCreateBlob(asm_code.size, disassembly)))
+        memcpy(ID3D10Blob_GetBufferPointer(*disassembly), asm_code.code, asm_code.size);
+
+    vkd3d_shader_free_shader_code(&asm_code);
+
+    return hr;
 }
 
 HRESULT WINAPI D3DCompileFromFile(const WCHAR *filename, const D3D_SHADER_MACRO *defines, ID3DInclude *include,
