@@ -206,6 +206,18 @@ static char* format_uuid(const UINT8 uuid[16], char out[UUID_STRING_LEN])
     return out;
 }
 
+static USHORT macho_cpu_to_machine(unsigned cpu)
+{
+    switch (cpu)
+    {
+    case MACHO_CPU_TYPE_X86:    return IMAGE_FILE_MACHINE_I386;
+    case MACHO_CPU_TYPE_X86_64: return IMAGE_FILE_MACHINE_AMD64;
+    default:
+        FIXME("Untranslated Mach-O CPU %x\n", cpu);
+        return IMAGE_FILE_MACHINE_UNKNOWN;
+    }
+}
+
 /******************************************************************
  *              macho_calc_range
  *
@@ -744,9 +756,9 @@ static BOOL macho_map_file(struct process *pcs, const WCHAR *filenameW,
     WCHAR*              filename;
     struct section_info info;
     BOOL                ret = FALSE;
-    UINT32 target_cpu = (pcs->is_64bit) ? MACHO_CPU_TYPE_X86_64 : MACHO_CPU_TYPE_X86;
-    UINT32 target_magic = (pcs->is_64bit) ? MACHO_MH_MAGIC_64 : MACHO_MH_MAGIC_32;
-    UINT32 target_cmd   = (pcs->is_64bit) ? MACHO_LC_SEGMENT_64 : MACHO_LC_SEGMENT;
+    UINT32 target_cpu = (pcs->is_host_64bit) ? MACHO_CPU_TYPE_X86_64 : MACHO_CPU_TYPE_X86;
+    UINT32 target_magic = (pcs->is_host_64bit) ? MACHO_MH_MAGIC_64 : MACHO_MH_MAGIC_32;
+    UINT32 target_cmd   = (pcs->is_host_64bit) ? MACHO_LC_SEGMENT_64 : MACHO_LC_SEGMENT;
     DWORD bytes_read;
 
     struct
@@ -762,8 +774,8 @@ static BOOL macho_map_file(struct process *pcs, const WCHAR *filenameW,
     ifm->modtype = DMT_MACHO;
     ifm->ops = &macho_file_map_ops;
     ifm->alternate = NULL;
-    ifm->addr_size = (pcs->is_64bit) ? 64 : 32;
-    fmap->header_size = (pcs->is_64bit) ? sizeof(struct macho_header) : FIELD_OFFSET(struct macho_header, reserved);
+    ifm->addr_size = (pcs->is_host_64bit) ? 64 : 32;
+    fmap->header_size = (pcs->is_host_64bit) ? sizeof(struct macho_header) : FIELD_OFFSET(struct macho_header, reserved);
 
     if (!(filename = get_dos_file_name(filenameW))) return FALSE;
 
@@ -1063,15 +1075,16 @@ static void macho_finish_stabs(struct module* module, struct hash_table* ht_symt
             {
             case SymTagFunction:
                 func = (struct symt_function*)sym;
-                if (func->address == module->format_info[DFI_MACHO]->u.macho_info->load_addr)
+                if (func->ranges[0].low == module->format_info[DFI_MACHO]->u.macho_info->load_addr)
                 {
-                    TRACE("Adjusting function %p/%s!%s from 0x%08Ix to 0x%08Ix\n", func,
+                    TRACE("Adjusting function %p/%s!%s from %#I64x to %#Ix\n", func,
                           debugstr_w(module->modulename), sym->hash_elt.name,
-                          func->address, ste->addr);
-                    func->address = ste->addr;
+                          func->ranges[0].low, ste->addr);
+                    func->ranges[0].high += ste->addr - func->ranges[0].low;
+                    func->ranges[0].low = ste->addr;
                     adjusted = TRUE;
                 }
-                if (func->address == ste->addr)
+                if (func->ranges[0].low == ste->addr)
                     ste->used = 1;
                 break;
             case SymTagData:
@@ -1338,8 +1351,8 @@ static BOOL image_uses_split_segs(struct process* process, ULONG_PTR load_addr)
 
     if (load_addr)
     {
-        UINT32 target_cpu = (process->is_64bit) ? MACHO_CPU_TYPE_X86_64 : MACHO_CPU_TYPE_X86;
-        UINT32 target_magic = (process->is_64bit) ? MACHO_MH_MAGIC_64 : MACHO_MH_MAGIC_32;
+        UINT32 target_cpu = (process->is_host_64bit) ? MACHO_CPU_TYPE_X86_64 : MACHO_CPU_TYPE_X86;
+        UINT32 target_magic = (process->is_host_64bit) ? MACHO_MH_MAGIC_64 : MACHO_MH_MAGIC_32;
         struct macho_header header;
 
         if (read_process_memory(process, load_addr, &header, FIELD_OFFSET(struct macho_header, reserved)) &&
@@ -1353,6 +1366,26 @@ static BOOL image_uses_split_segs(struct process* process, ULONG_PTR load_addr)
     return split_segs;
 }
 
+/******************************************************************
+ *              image_get_machine
+ *
+ * For a module identified by its load address, return the machine field
+ * of the (loaded) Macho header.
+ */
+static USHORT image_get_machine(struct process *process, ULONG_PTR load_addr)
+{
+    if (load_addr)
+    {
+        struct macho_header header;
+        UINT32 target_magic = (process->is_host_64bit) ? MACHO_MH_MAGIC_64 : MACHO_MH_MAGIC_32;
+
+        if (read_process_memory(process, load_addr, &header, FIELD_OFFSET(struct macho_header, reserved)) &&
+            header.magic == target_magic)
+            return macho_cpu_to_machine(header.cputype);
+    }
+    return IMAGE_FILE_MACHINE_UNKNOWN;
+}
+
 /******************************************************************
  *              macho_load_debug_info
  *
@@ -1478,7 +1511,7 @@ static BOOL macho_load_file(struct process* pcs, const WCHAR* filename,
             load_addr = fmap.u.macho.segs_start;
         macho_info->module = module_new(pcs, filename, DMT_MACHO, FALSE, load_addr,
                                         fmap.u.macho.segs_size, 0, calc_crc32(fmap.u.macho.handle),
-                                        IMAGE_FILE_MACHINE_UNKNOWN);
+                                        image_get_machine(pcs, load_addr));
         if (!macho_info->module)
         {
             HeapFree(GetProcessHeap(), 0, modfmt);
@@ -1582,7 +1615,7 @@ static BOOL macho_search_and_load_file(struct process* pcs, const WCHAR* filenam
         ret = search_unix_path(p, fallback, macho_load_file_cb, &load_params);
     }
     if (!ret && p == filename)
-        ret = search_dll_path(pcs, filename, macho_load_file_cb, &load_params);
+        ret = search_dll_path(pcs, filename, IMAGE_FILE_MACHINE_UNKNOWN, macho_load_file_cb, &load_params);
 
     return ret;
 }
@@ -1607,14 +1640,14 @@ static BOOL macho_enum_modules_internal(const struct process* pcs,
     TRACE("(%p/%p, %s, %p, %p)\n", pcs, pcs->handle, debugstr_w(main_name), cb,
             user);
 
-    if (pcs->is_64bit)
+    if (pcs->is_host_64bit)
         len = sizeof(image_infos.infos64);
     else
         len = sizeof(image_infos.infos32);
     if (!pcs->dbg_hdr_addr ||
         !read_process_memory(pcs, pcs->dbg_hdr_addr, &image_infos, len))
         goto done;
-    if (!pcs->is_64bit)
+    if (!pcs->is_host_64bit)
     {
         struct dyld_all_image_infos32 temp = image_infos.infos32;
         image_infos.infos64.infoArrayCount = temp.infoArrayCount;
@@ -1624,7 +1657,7 @@ static BOOL macho_enum_modules_internal(const struct process* pcs,
         goto done;
     TRACE("Process has %u image infos at %I64x\n", image_infos.infos64.infoArrayCount, image_infos.infos64.infoArray);
 
-    if (pcs->is_64bit)
+    if (pcs->is_host_64bit)
         len = sizeof(info_array->info64);
     else
         len = sizeof(info_array->info32);
@@ -1638,7 +1671,7 @@ static BOOL macho_enum_modules_internal(const struct process* pcs,
     for (i = 0; i < image_infos.infos64.infoArrayCount; i++)
     {
         struct dyld_image_info64 info;
-        if (pcs->is_64bit)
+        if (pcs->is_host_64bit)
             info = info_array[i].info64;
         else
         {
@@ -1828,13 +1861,13 @@ static BOOL macho_search_loader(struct process* pcs, struct macho_info* macho_in
     char path[1024];
     BOOL got_path = FALSE;
 
-    if (pcs->is_64bit)
+    if (pcs->is_host_64bit)
         len = sizeof(image_infos.infos64);
     else
         len = sizeof(image_infos.infos32);
     if (read_process_memory(pcs, pcs->dbg_hdr_addr, &image_infos, len))
     {
-        if (pcs->is_64bit)
+        if (pcs->is_host_64bit)
             len = sizeof(image_info.info64);
         else
         {
@@ -1846,7 +1879,7 @@ static BOOL macho_search_loader(struct process* pcs, struct macho_info* macho_in
         if (image_infos.infos64.infoArray && image_infos.infos64.infoArrayCount &&
             read_process_memory(pcs, image_infos.infos64.infoArray, &image_info, len))
         {
-            if (!pcs->is_64bit)
+            if (!pcs->is_host_64bit)
             {
                 struct dyld_image_info32 temp = image_info.info32;
                 image_info.info64.imageLoadAddress = temp.imageLoadAddress;
