@@ -136,6 +136,7 @@ static void test_interfaces(const GUID *clsid, const GUID *iid)
     {
         &IID_IMediaObject,
         &IID_IMediaObjectInPlace,
+        &IID_IMediaParams,
         &IID_IMediaParamInfo,
     };
     IUnknown *unk, *unk2, *unk3;
@@ -307,11 +308,14 @@ static void test_chorus_parameters(void)
 
     hr = CoCreateInstance(&GUID_DSFX_STANDARD_CHORUS, NULL, CLSCTX_INPROC_SERVER,
             &IID_IDirectSoundFXChorus, (void **)&chorus);
-    todo_wine ok(hr == S_OK, "Got hr %#lx.\n", hr);
+    ok(hr == S_OK, "Got hr %#lx.\n", hr);
     if (hr != S_OK)
         return;
 
     hr = IDirectSoundFXChorus_GetAllParameters(chorus, &params);
+    todo_wine ok(hr == S_OK, "Got hr %#lx.\n", hr);
+    if (hr != S_OK)
+        return;
     ok(hr == S_OK, "Got hr %#lx.\n", hr);
     ok(params.fWetDryMix == 50.0f, "Got wetness %.8e%%.\n", params.fWetDryMix);
     ok(params.fDepth == 10.0f, "Got depth %.8e.\n", params.fDepth);
@@ -334,12 +338,12 @@ static void test_compressor_parameters(void)
 
     hr = CoCreateInstance(&GUID_DSFX_STANDARD_COMPRESSOR, NULL, CLSCTX_INPROC_SERVER,
             &IID_IDirectSoundFXCompressor, (void **)&compressor);
+    ok(hr == S_OK, "Got hr %#lx.\n", hr);
+
+    hr = IDirectSoundFXCompressor_GetAllParameters(compressor, &params);
     todo_wine ok(hr == S_OK, "Got hr %#lx.\n", hr);
     if (hr != S_OK)
         return;
-
-    hr = IDirectSoundFXCompressor_GetAllParameters(compressor, &params);
-    ok(hr == S_OK, "Got hr %#lx.\n", hr);
     ok(params.fGain == 0.0f, "Got gain %.8e dB.\n", params.fGain);
     ok(params.fAttack == 10.0f, "Got attack time %.8e ms.\n", params.fAttack);
     ok(params.fThreshold == -20.0f, "Got threshold %.8e dB.\n", params.fThreshold);
@@ -384,12 +388,14 @@ static void test_echo_parameters(void)
 
     hr = CoCreateInstance(&GUID_DSFX_STANDARD_ECHO, NULL, CLSCTX_INPROC_SERVER,
             &IID_IDirectSoundFXEcho, (void **)&echo);
-    todo_wine ok(hr == S_OK, "Got hr %#lx.\n", hr);
+    ok(hr == S_OK, "Got hr %#lx.\n", hr);
     if (hr != S_OK)
         return;
 
     hr = IDirectSoundFXEcho_GetAllParameters(echo, &params);
-    ok(hr == S_OK, "Got hr %#lx.\n", hr);
+    todo_wine ok(hr == S_OK, "Got hr %#lx.\n", hr);
+    if (hr != S_OK)
+        return;
     ok(params.fWetDryMix == 50.0f, "Got %.8e%% wetness.\n", params.fWetDryMix);
     ok(params.fFeedback == 50.0f, "Got %.8e%% feedback.\n", params.fFeedback);
     ok(params.fLeftDelay == 500.0f, "Got left delay %.8e ms.\n", params.fLeftDelay);
@@ -409,11 +415,14 @@ static void test_flanger_parameters(void)
 
     hr = CoCreateInstance(&GUID_DSFX_STANDARD_FLANGER, NULL, CLSCTX_INPROC_SERVER,
             &IID_IDirectSoundFXFlanger, (void **)&flanger);
-    todo_wine ok(hr == S_OK, "Got hr %#lx.\n", hr);
+    ok(hr == S_OK, "Got hr %#lx.\n", hr);
     if (hr != S_OK)
         return;
 
     hr = IDirectSoundFXFlanger_GetAllParameters(flanger, &params);
+    todo_wine ok(hr == S_OK, "Got hr %#lx.\n", hr);
+    if (hr != S_OK)
+        return;
     ok(hr == S_OK, "Got hr %#lx.\n", hr);
     ok(params.fWetDryMix == 50.0f, "Got %.8e%% wetness.\n", params.fWetDryMix);
     ok(params.fDepth == 100.0f, "Got %.8e * 0.01%% depth.\n", params.fDepth);
@@ -541,11 +550,11 @@ START_TEST(dsdmo)
     }
     tests[] =
     {
-        {&GUID_DSFX_STANDARD_CHORUS,        &IID_IDirectSoundFXChorus, TRUE},
-        {&GUID_DSFX_STANDARD_COMPRESSOR,    &IID_IDirectSoundFXCompressor, TRUE},
+        {&GUID_DSFX_STANDARD_CHORUS,        &IID_IDirectSoundFXChorus},
+        {&GUID_DSFX_STANDARD_COMPRESSOR,    &IID_IDirectSoundFXCompressor},
         {&GUID_DSFX_STANDARD_DISTORTION,    &IID_IDirectSoundFXDistortion, TRUE},
-        {&GUID_DSFX_STANDARD_ECHO,          &IID_IDirectSoundFXEcho, TRUE},
-        {&GUID_DSFX_STANDARD_FLANGER,       &IID_IDirectSoundFXFlanger, TRUE},
+        {&GUID_DSFX_STANDARD_ECHO,          &IID_IDirectSoundFXEcho},
+        {&GUID_DSFX_STANDARD_FLANGER,       &IID_IDirectSoundFXFlanger},
         {&GUID_DSFX_STANDARD_GARGLE,        &IID_IDirectSoundFXGargle, TRUE},
         {&GUID_DSFX_STANDARD_I3DL2REVERB,   &IID_IDirectSoundFXI3DL2Reverb},
         {&GUID_DSFX_STANDARD_PARAMEQ,       &IID_IDirectSoundFXParamEq},
