@@ -39,17 +39,15 @@ WINE_DEFAULT_DEBUG_CHANNEL(gdiplus);
  *
  * and puts the output in out.
  * */
-static void matrix_multiply(GDIPCONST REAL * left, GDIPCONST REAL * right, REAL * out)
+static inline void matrix_multiply(GDIPCONST REAL * left, GDIPCONST REAL * right, REAL * out)
 {
     REAL temp[6];
-    int i, odd;
-
-    for(i = 0; i < 6; i++){
-        odd = i % 2;
-        temp[i] = left[i - odd] * right[odd] + left[i - odd + 1] * right[odd + 2] +
-                  (i >= 4 ? right[odd + 4] : 0.0);
-    }
-
+    temp[0] = left[0] * right[0] + left[1] * right[2];
+    temp[1] = left[0] * right[1] + left[1] * right[3];
+    temp[2] = left[2] * right[0] + left[3] * right[2];
+    temp[3] = left[2] * right[1] + left[3] * right[3];
+    temp[4] = left[4] * right[0] + left[5] * right[2] + right[4];
+    temp[5] = left[4] * right[1] + left[5] * right[3] + right[5];
     memcpy(out, temp, 6 * sizeof(REAL));
 }
 
@@ -122,7 +120,7 @@ GpStatus WINGDIPAPI GdipCreateMatrix3I(GDIPCONST GpRect *rect, GDIPCONST GpPoint
 
 GpStatus WINGDIPAPI GdipCloneMatrix(GpMatrix *matrix, GpMatrix **clone)
 {
-    TRACE("(%p, %p)\n", matrix, clone);
+    TRACE("(%s, %p)\n", debugstr_matrix(matrix), clone);
 
     if(!matrix || !clone)
         return InvalidParameter;
@@ -170,7 +168,7 @@ GpStatus WINGDIPAPI GdipDeleteMatrix(GpMatrix *matrix)
 GpStatus WINGDIPAPI GdipGetMatrixElements(GDIPCONST GpMatrix *matrix,
     REAL *out)
 {
-    TRACE("(%p, %p)\n", matrix, out);
+    TRACE("(%s, %p)\n", debugstr_matrix(matrix), out);
 
     if(!matrix || !out)
         return InvalidParameter;
@@ -184,45 +182,48 @@ GpStatus WINGDIPAPI GdipInvertMatrix(GpMatrix *matrix)
 {
     GpMatrix copy;
     REAL det;
-    BOOL invertible;
 
-    TRACE("(%p)\n", matrix);
+    TRACE("(%s)\n", debugstr_matrix(matrix));
 
     if(!matrix)
         return InvalidParameter;
 
-    GdipIsMatrixInvertible(matrix, &invertible);
-    if(!invertible)
-        return InvalidParameter;
-
     /* optimize inverting simple scaling and translation matrices */
     if(matrix->matrix[1] == 0 && matrix->matrix[2] == 0)
     {
-        matrix->matrix[4] = -matrix->matrix[4] / matrix->matrix[0];
-        matrix->matrix[5] = -matrix->matrix[5] / matrix->matrix[3];
-        matrix->matrix[0] = 1 / matrix->matrix[0];
-        matrix->matrix[3] = 1 / matrix->matrix[3];
+        if (matrix->matrix[0] != 0 && matrix->matrix[3] != 0)
+        {
+            matrix->matrix[4] = -matrix->matrix[4] / matrix->matrix[0];
+            matrix->matrix[5] = -matrix->matrix[5] / matrix->matrix[3];
+            matrix->matrix[0] = 1 / matrix->matrix[0];
+            matrix->matrix[3] = 1 / matrix->matrix[3];
 
-        return Ok;
+            return Ok;
+        }
+        else
+            return InvalidParameter;
     }
-
     det = matrix_det(matrix);
+    if (!(fabs(det) >= 1e-5))
+        return InvalidParameter;
+
+    det = 1 / det;
 
     copy = *matrix;
     /* store result */
-    matrix->matrix[0] =   copy.matrix[3] / det;
-    matrix->matrix[1] =  -copy.matrix[1] / det;
-    matrix->matrix[2] =  -copy.matrix[2] / det;
-    matrix->matrix[3] =   copy.matrix[0] / det;
-    matrix->matrix[4] =  (copy.matrix[2]*copy.matrix[5]-copy.matrix[3]*copy.matrix[4]) / det;
-    matrix->matrix[5] = -(copy.matrix[0]*copy.matrix[5]-copy.matrix[1]*copy.matrix[4]) / det;
+    matrix->matrix[0] =   copy.matrix[3] * det;
+    matrix->matrix[1] =  -copy.matrix[1] * det;
+    matrix->matrix[2] =  -copy.matrix[2] * det;
+    matrix->matrix[3] =   copy.matrix[0] * det;
+    matrix->matrix[4] =  (copy.matrix[2]*copy.matrix[5]-copy.matrix[3]*copy.matrix[4]) * det;
+    matrix->matrix[5] = -(copy.matrix[0]*copy.matrix[5]-copy.matrix[1]*copy.matrix[4]) * det;
 
     return Ok;
 }
 
 GpStatus WINGDIPAPI GdipIsMatrixInvertible(GDIPCONST GpMatrix *matrix, BOOL *result)
 {
-    TRACE("(%p, %p)\n", matrix, result);
+    TRACE("(%s, %p)\n", debugstr_matrix(matrix), result);
 
     if(!matrix || !result)
         return InvalidParameter;
@@ -238,7 +239,7 @@ GpStatus WINGDIPAPI GdipIsMatrixInvertible(GDIPCONST GpMatrix *matrix, BOOL *res
 GpStatus WINGDIPAPI GdipMultiplyMatrix(GpMatrix *matrix, GDIPCONST GpMatrix* matrix2,
     GpMatrixOrder order)
 {
-    TRACE("(%p, %p, %d)\n", matrix, matrix2, order);
+    TRACE("(%s, %s, %d)\n", debugstr_matrix(matrix), debugstr_matrix(matrix2), order);
 
     if(!matrix || !matrix2)
         return InvalidParameter;
@@ -258,7 +259,7 @@ GpStatus WINGDIPAPI GdipRotateMatrix(GpMatrix *matrix, REAL angle,
 {
     REAL cos_theta, sin_theta, rotate[6];
 
-    TRACE("(%p, %.2f, %d)\n", matrix, angle, order);
+    TRACE("(%p, %.2f, %d)\n", debugstr_matrix(matrix), angle, order);
 
     if(!matrix)
         return InvalidParameter;
@@ -287,24 +288,27 @@ GpStatus WINGDIPAPI GdipRotateMatrix(GpMatrix *matrix, REAL angle,
 GpStatus WINGDIPAPI GdipScaleMatrix(GpMatrix *matrix, REAL scaleX, REAL scaleY,
     GpMatrixOrder order)
 {
-    REAL scale[6];
-
-    TRACE("(%p, %.2f, %.2f, %d)\n", matrix, scaleX, scaleY, order);
+    TRACE("(%s, %.2f, %.2f, %d)\n", debugstr_matrix(matrix), scaleX, scaleY, order);
 
     if(!matrix)
         return InvalidParameter;
 
-    scale[0] = scaleX;
-    scale[1] = 0.0;
-    scale[2] = 0.0;
-    scale[3] = scaleY;
-    scale[4] = 0.0;
-    scale[5] = 0.0;
-
     if(order == MatrixOrderAppend)
-        matrix_multiply(matrix->matrix, scale, matrix->matrix);
+    {
+        matrix->matrix[0] *= scaleX;
+        matrix->matrix[1] *= scaleY;
+        matrix->matrix[2] *= scaleX;
+        matrix->matrix[3] *= scaleY;
+        matrix->matrix[4] *= scaleX;
+        matrix->matrix[5] *= scaleY;
+    }
     else if (order == MatrixOrderPrepend)
-        matrix_multiply(scale, matrix->matrix, matrix->matrix);
+    {
+        matrix->matrix[0] *= scaleX;
+        matrix->matrix[1] *= scaleX;
+        matrix->matrix[2] *= scaleY;
+        matrix->matrix[3] *= scaleY;
+    }
     else
         return InvalidParameter;
 
@@ -314,7 +318,7 @@ GpStatus WINGDIPAPI GdipScaleMatrix(GpMatrix *matrix, REAL scaleX, REAL scaleY,
 GpStatus WINGDIPAPI GdipSetMatrixElements(GpMatrix *matrix, REAL m11, REAL m12,
     REAL m21, REAL m22, REAL dx, REAL dy)
 {
-    TRACE("(%p, %.2f, %.2f, %.2f, %.2f, %.2f, %.2f)\n", matrix, m11, m12,
+    TRACE("(%s, %.2f, %.2f, %.2f, %.2f, %.2f, %.2f)\n", debugstr_matrix(matrix), m11, m12,
             m21, m22, dx, dy);
 
     if(!matrix)
@@ -335,7 +339,7 @@ GpStatus WINGDIPAPI GdipShearMatrix(GpMatrix *matrix, REAL shearX, REAL shearY,
 {
     REAL shear[6];
 
-    TRACE("(%p, %.2f, %.2f, %d)\n", matrix, shearX, shearY, order);
+    TRACE("(%s, %.2f, %.2f, %d)\n", debugstr_matrix(matrix), shearX, shearY, order);
 
     if(!matrix)
         return InvalidParameter;
@@ -364,7 +368,7 @@ GpStatus WINGDIPAPI GdipTransformMatrixPoints(GpMatrix *matrix, GpPointF *pts,
     REAL x, y;
     INT i;
 
-    TRACE("(%p, %p, %d)\n", matrix, pts, count);
+    TRACE("(%s, %p, %d)\n", debugstr_matrix(matrix), pts, count);
 
     if(!matrix || !pts || count <= 0)
         return InvalidParameter;
@@ -387,7 +391,7 @@ GpStatus WINGDIPAPI GdipTransformMatrixPointsI(GpMatrix *matrix, GpPoint *pts, I
     GpStatus ret;
     INT i;
 
-    TRACE("(%p, %p, %d)\n", matrix, pts, count);
+    TRACE("(%s, %p, %d)\n", debugstr_matrix(matrix), pts, count);
 
     if(count <= 0)
         return InvalidParameter;
@@ -416,24 +420,23 @@ GpStatus WINGDIPAPI GdipTransformMatrixPointsI(GpMatrix *matrix, GpPoint *pts, I
 GpStatus WINGDIPAPI GdipTranslateMatrix(GpMatrix *matrix, REAL offsetX,
     REAL offsetY, GpMatrixOrder order)
 {
-    REAL translate[6];
-
-    TRACE("(%p, %.2f, %.2f, %d)\n", matrix, offsetX, offsetY, order);
+    TRACE("(%s, %.2f, %.2f, %d)\n", debugstr_matrix(matrix), offsetX, offsetY, order);
 
     if(!matrix)
         return InvalidParameter;
 
-    translate[0] = 1.0;
-    translate[1] = 0.0;
-    translate[2] = 0.0;
-    translate[3] = 1.0;
-    translate[4] = offsetX;
-    translate[5] = offsetY;
-
     if(order == MatrixOrderAppend)
-        matrix_multiply(matrix->matrix, translate, matrix->matrix);
+    {
+        matrix->matrix[4] += offsetX;
+        matrix->matrix[5] += offsetY;
+    }
     else if (order == MatrixOrderPrepend)
-        matrix_multiply(translate, matrix->matrix, matrix->matrix);
+    {
+        matrix->matrix[4] = offsetX * matrix->matrix[0] + offsetY * matrix->matrix[2]
+            + matrix->matrix[4];
+        matrix->matrix[5] = offsetX * matrix->matrix[1] + offsetY * matrix->matrix[3]
+            + matrix->matrix[5];
+    }
     else
         return InvalidParameter;
 
@@ -445,7 +448,7 @@ GpStatus WINGDIPAPI GdipVectorTransformMatrixPoints(GpMatrix *matrix, GpPointF *
     REAL x, y;
     INT i;
 
-    TRACE("(%p, %p, %d)\n", matrix, pts, count);
+    TRACE("(%s, %p, %d)\n", debugstr_matrix(matrix), pts, count);
 
     if(!matrix || !pts || count <= 0)
         return InvalidParameter;
@@ -468,7 +471,7 @@ GpStatus WINGDIPAPI GdipVectorTransformMatrixPointsI(GpMatrix *matrix, GpPoint *
     GpStatus ret;
     INT i;
 
-    TRACE("(%p, %p, %d)\n", matrix, pts, count);
+    TRACE("(%s, %p, %d)\n", debugstr_matrix(matrix), pts, count);
 
     if(count <= 0)
         return InvalidParameter;
@@ -497,7 +500,7 @@ GpStatus WINGDIPAPI GdipVectorTransformMatrixPointsI(GpMatrix *matrix, GpPoint *
 GpStatus WINGDIPAPI GdipIsMatrixEqual(GDIPCONST GpMatrix *matrix, GDIPCONST GpMatrix *matrix2,
     BOOL *result)
 {
-    TRACE("(%p, %p, %p)\n", matrix, matrix2, result);
+    TRACE("(%s, %s, %p)\n", debugstr_matrix(matrix), debugstr_matrix(matrix2), result);
 
     if(!matrix || !matrix2 || !result)
         return InvalidParameter;
@@ -516,7 +519,7 @@ GpStatus WINGDIPAPI GdipIsMatrixIdentity(GDIPCONST GpMatrix *matrix, BOOL *resul
         0.0, 0.0 }
     };
 
-    TRACE("(%p, %p)\n", matrix, result);
+    TRACE("(%s, %p)\n", debugstr_matrix(matrix), result);
 
     if(!matrix || !result)
         return InvalidParameter;
