@@ -24,8 +24,6 @@
 #include <stdlib.h>
 #include <stdarg.h>
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #include "windef.h"
@@ -36,6 +34,7 @@
 #include "winnt.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(seh);
+WINE_DECLARE_DEBUG_CHANNEL(relay);
 WINE_DECLARE_DEBUG_CHANNEL(threadname);
 
 typedef struct _SCOPE_TABLE
@@ -105,7 +104,7 @@ __ASM_GLOBAL_FUNC( __chkstk, "lsl r4, r4, #2\n\t"
 /***********************************************************************
  *		RtlCaptureContext (NTDLL.@)
  */
-__ASM_STDCALL_FUNC( RtlCaptureContext, 4,
+__ASM_GLOBAL_FUNC( RtlCaptureContext,
                     "str r1, [r0, #0x8]\n\t"   /* context->R1 */
                     "mov r1, #0x0200000\n\t"   /* CONTEXT_ARM */
                     "add r1, r1, #0x7\n\t"     /* CONTEXT_CONTROL|CONTEXT_INTEGER|CONTEXT_FLOATING_POINT */
@@ -165,7 +164,7 @@ static NTSTATUS virtual_unwind( ULONG type, DISPATCHER_CONTEXT *dispatch, CONTEX
     {
         struct unwind_builtin_dll_params params = { type, dispatch, context };
 
-        status = NTDLL_UNIX_CALL( unwind_builtin_dll, &params );
+        status = WINE_UNIX_CALL( unix_unwind_builtin_dll, &params );
         if (status != STATUS_SUCCESS) return status;
 
         if (dispatch->EstablisherFrame)
@@ -463,7 +462,7 @@ NTSTATUS WINAPI KiUserExceptionDispatcher( EXCEPTION_RECORD *rec, CONTEXT *conte
     TRACE( "code=%lx flags=%lx addr=%p pc=%08lx\n",
            rec->ExceptionCode, rec->ExceptionFlags, rec->ExceptionAddress, context->Pc );
     for (c = 0; c < rec->NumberParameters; c++)
-        TRACE( " info[%ld]=%08lx\n", c, rec->ExceptionInformation[c] );
+        TRACE( " info[%ld]=%08Ix\n", c, rec->ExceptionInformation[c] );
 
     if (rec->ExceptionCode == EXCEPTION_WINE_STUB)
     {
@@ -472,13 +471,13 @@ NTSTATUS WINAPI KiUserExceptionDispatcher( EXCEPTION_RECORD *rec, CONTEXT *conte
                      rec->ExceptionAddress,
                      (char*)rec->ExceptionInformation[0], (char*)rec->ExceptionInformation[1] );
         else
-            MESSAGE( "wine: Call from %p to unimplemented function %s.%ld, aborting\n",
+            MESSAGE( "wine: Call from %p to unimplemented function %s.%Id, aborting\n",
                      rec->ExceptionAddress,
                      (char*)rec->ExceptionInformation[0], rec->ExceptionInformation[1] );
     }
     else if (rec->ExceptionCode == EXCEPTION_WINE_NAME_THREAD && rec->ExceptionInformation[0] == 0x1000)
     {
-        if ((DWORD)rec->ExceptionInformation[2] == -1)
+        if ((DWORD)rec->ExceptionInformation[2] == -1 || (DWORD)rec->ExceptionInformation[2] == GetCurrentThreadId())
             WARN_(threadname)( "Thread renamed to %s\n", debugstr_a((char *)rec->ExceptionInformation[1]) );
         else
             WARN_(threadname)( "Thread ID %04lx renamed to %s\n", (DWORD)rec->ExceptionInformation[2],
@@ -685,7 +684,7 @@ static void pop_fpregs_range( int first, int last, CONTEXT *context,
     for (i = first; i <= last; i++)
     {
         if (ptrs && i >= 8 && i <= 15) (&ptrs->D8)[i - 8] = (ULONGLONG *)context->Sp;
-        context->u.D[i] = *(ULONGLONG *)context->Sp;
+        context->D[i] = *(ULONGLONG *)context->Sp;
         context->Sp += 8;
     }
 }
@@ -781,32 +780,32 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
                                  CONTEXT *context, KNONVOLATILE_CONTEXT_POINTERS *ptrs )
 {
     int i, pos = 0;
-    int pf = 0, ef = 0, fpoffset = 0, stack = func->u.s.StackAdjust;
+    int pf = 0, ef = 0, fpoffset = 0, stack = func->StackAdjust;
     int prologue_regmask = 0;
     int epilogue_regmask = 0;
     unsigned int offset, len;
     BYTE prologue[10], *prologue_end, epilogue[20], *epilogue_end;
 
     TRACE( "function %lx-%lx: len=%#x flag=%x ret=%u H=%u reg=%u R=%u L=%u C=%u stackadjust=%x\n",
-           base + func->BeginAddress, base + func->BeginAddress + func->u.s.FunctionLength * 2,
-           func->u.s.FunctionLength, func->u.s.Flag, func->u.s.Ret,
-           func->u.s.H, func->u.s.Reg, func->u.s.R, func->u.s.L, func->u.s.C, func->u.s.StackAdjust );
+           base + func->BeginAddress, base + func->BeginAddress + func->FunctionLength * 2,
+           func->FunctionLength, func->Flag, func->Ret,
+           func->H, func->Reg, func->R, func->L, func->C, func->StackAdjust );
 
     offset = (pc - base) - func->BeginAddress;
-    if (func->u.s.StackAdjust >= 0x03f4)
+    if (func->StackAdjust >= 0x03f4)
     {
-        pf = func->u.s.StackAdjust & 0x04;
-        ef = func->u.s.StackAdjust & 0x08;
-        stack = (func->u.s.StackAdjust & 3) + 1;
+        pf = func->StackAdjust & 0x04;
+        ef = func->StackAdjust & 0x08;
+        stack = (func->StackAdjust & 3) + 1;
     }
 
-    if (!func->u.s.R || pf)
+    if (!func->R || pf)
     {
-        int first = 4, last = func->u.s.Reg + 4;
+        int first = 4, last = func->Reg + 4;
         if (pf)
         {
-            first = (~func->u.s.StackAdjust) & 3;
-            if (func->u.s.R)
+            first = (~func->StackAdjust) & 3;
+            if (func->R)
                 last = 3;
         }
         for (i = first; i <= last; i++)
@@ -814,31 +813,31 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
         fpoffset = last + 1 - first;
     }
 
-    if (!func->u.s.R || ef)
+    if (!func->R || ef)
     {
-        int first = 4, last = func->u.s.Reg + 4;
+        int first = 4, last = func->Reg + 4;
         if (ef)
         {
-            first = (~func->u.s.StackAdjust) & 3;
-            if (func->u.s.R)
+            first = (~func->StackAdjust) & 3;
+            if (func->R)
                 last = 3;
         }
         for (i = first; i <= last; i++)
             epilogue_regmask |= 1 << i;
     }
 
-    if (func->u.s.C)
+    if (func->C)
     {
         prologue_regmask |= 1 << 11;
         epilogue_regmask |= 1 << 11;
     }
 
-    if (func->u.s.L)
+    if (func->L)
     {
         prologue_regmask |= 1 << 14; /* lr */
-        if (func->u.s.Ret != 0)
+        if (func->Ret != 0)
             epilogue_regmask |= 1 << 14; /* lr */
-        else if (!func->u.s.H)
+        else if (!func->H)
             epilogue_regmask |= 1 << 15; /* pc */
     }
 
@@ -856,12 +855,12 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
         }
     }
 
-    if (func->u.s.R && func->u.s.Reg != 7)
-        prologue[pos++] = 0xe0 | func->u.s.Reg; /* vpush {d8-dX} */
+    if (func->R && func->Reg != 7)
+        prologue[pos++] = 0xe0 | func->Reg; /* vpush {d8-dX} */
 
-    if (func->u.s.C && fpoffset == 0)
+    if (func->C && fpoffset == 0)
         prologue[pos++] = 0xfb; /* mov r11, sp - handled as nop16 */
-    else if (func->u.s.C)
+    else if (func->C)
         prologue[pos++] = 0xfc; /* add r11, sp, #x - handled as nop32 */
 
     if (prologue_regmask & 0xf00) /* r8-r11 set */
@@ -881,7 +880,7 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
         prologue[pos++] = bitmask & 0xff;
     }
 
-    if (func->u.s.H)
+    if (func->H)
         prologue[pos++] = 0x04; /* push {r0-r3} - handled as sub sp, sp, #16 */
 
     prologue[pos++] = 0xff; /* end */
@@ -902,8 +901,8 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
         }
     }
 
-    if (func->u.s.R && func->u.s.Reg != 7)
-        epilogue[pos++] = 0xe0 | func->u.s.Reg; /* vpush {d8-dX} */
+    if (func->R && func->Reg != 7)
+        epilogue[pos++] = 0xe0 | func->Reg; /* vpush {d8-dX} */
 
     if (epilogue_regmask & 0x7f00) /* r8-r11, lr set */
     {
@@ -922,23 +921,23 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
         epilogue[pos++] = bitmask & 0xff;
     }
 
-    if (func->u.s.H && !(func->u.s.L && func->u.s.Ret == 0))
+    if (func->H && !(func->L && func->Ret == 0))
         epilogue[pos++] = 0x04; /* add sp, sp, #16 */
-    else if (func->u.s.H && (func->u.s.L && func->u.s.Ret == 0))
+    else if (func->H && (func->L && func->Ret == 0))
     {
         epilogue[pos++] = 0xef; /* ldr lr, [sp], #20 */
         epilogue[pos++] = 5;
     }
 
-    if (func->u.s.Ret == 1)
+    if (func->Ret == 1)
         epilogue[pos++] = 0xfd; /* bx lr */
-    else if (func->u.s.Ret == 2)
+    else if (func->Ret == 2)
         epilogue[pos++] = 0xfe; /* b address */
     else
         epilogue[pos++] = 0xff; /* end */
     epilogue_end = &epilogue[pos];
 
-    if (func->u.s.Flag == 1 && offset < 4 * (prologue_end - prologue)) {
+    if (func->Flag == 1 && offset < 4 * (prologue_end - prologue)) {
         /* Check prologue */
         len = get_sequence_len( prologue, prologue_end, 0 );
         if (offset < len)
@@ -948,12 +947,12 @@ static void *unwind_packed_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION
         }
     }
 
-    if (func->u.s.Ret != 3 && 2 * func->u.s.FunctionLength - offset <= 4 * (epilogue_end - epilogue)) {
+    if (func->Ret != 3 && 2 * func->FunctionLength - offset <= 4 * (epilogue_end - epilogue)) {
         /* Check epilogue */
         len = get_sequence_len( epilogue, epilogue_end, 1 );
-        if (offset >= 2 * func->u.s.FunctionLength - len)
+        if (offset >= 2 * func->FunctionLength - len)
         {
-            process_unwind_codes( epilogue, epilogue_end, context, ptrs, offset - (2 * func->u.s.FunctionLength - len) );
+            process_unwind_codes( epilogue, epilogue_end, context, ptrs, offset - (2 * func->FunctionLength - len) );
             return NULL;
         }
     }
@@ -977,7 +976,7 @@ static void *unwind_full_data( ULONG_PTR base, ULONG_PTR pc, RUNTIME_FUNCTION *f
     void *data;
     BYTE *end;
 
-    info = (struct unwind_info *)((char *)base + func->u.UnwindData);
+    info = (struct unwind_info *)((char *)base + func->UnwindData);
     data = info + 1;
     epilogs = info->epilog;
     codes = info->codes;
@@ -1061,12 +1060,12 @@ PVOID WINAPI RtlVirtualUnwind( ULONG type, ULONG_PTR base, ULONG_PTR pc,
 {
     void *handler;
 
-    TRACE( "type %lx pc %lx sp %lx func %lx\n", type, pc, context->Sp, base + func->BeginAddress );
+    TRACE( "type %lx pc %Ix sp %lx func %lx\n", type, pc, context->Sp, base + func->BeginAddress );
 
     *handler_data = NULL;
 
     context->Pc = 0;
-    if (func->u.s.Flag)
+    if (func->Flag)
         handler = unwind_packed_data( base, pc, func, context, ctx_ptr );
     else
         handler = unwind_full_data( base, pc, func, context, handler_data, ctx_ptr );
@@ -1171,7 +1170,7 @@ void CDECL RtlRestoreContext( CONTEXT *context, EXCEPTION_RECORD *rec )
         context->Fpscr   = jmp->Fpscr;
 
         for (i = 0; i < 8; i++)
-            context->u.D[8+i] = jmp->D[i];
+            context->D[8+i] = jmp->D[i];
     }
     else if (rec && rec->ExceptionCode == STATUS_UNWIND_CONSOLIDATE && rec->NumberParameters >= 1)
     {
@@ -1226,7 +1225,7 @@ void WINAPI RtlUnwindEx( PVOID end_frame, PVOID target_ip, EXCEPTION_RECORD *rec
     TRACE( "code=%lx flags=%lx end_frame=%p target_ip=%p pc=%08lx\n",
            rec->ExceptionCode, rec->ExceptionFlags, end_frame, target_ip, context->Pc );
     for (i = 0; i < min( EXCEPTION_MAXIMUM_PARAMETERS, rec->NumberParameters ); i++)
-        TRACE( " info[%ld]=%016lx\n", i, rec->ExceptionInformation[i] );
+        TRACE( " info[%ld]=%08Ix\n", i, rec->ExceptionInformation[i] );
     TRACE("  r0=%08lx  r1=%08lx  r2=%08lx  r3=%08lx\n",
           context->R0, context->R1, context->R2, context->R3 );
     TRACE("  r4=%08lx  r5=%08lx  r6=%08lx  r7=%08lx\n",
@@ -1453,7 +1452,7 @@ EXCEPTION_DISPOSITION WINAPI __C_specific_handler( EXCEPTION_RECORD *rec,
 /***********************************************************************
  *		RtlRaiseException (NTDLL.@)
  */
-__ASM_STDCALL_FUNC( RtlRaiseException, 4,
+__ASM_GLOBAL_FUNC( RtlRaiseException,
                     "push {r0, lr}\n\t"
                     __ASM_EHABI(".save {r0, lr}\n\t")
                     __ASM_SEH(".seh_save_regs {r0, lr}\n\t")
@@ -1490,21 +1489,39 @@ USHORT WINAPI RtlCaptureStackBackTrace( ULONG skip, ULONG count, PVOID *buffer,
 }
 
 /***********************************************************************
- *           signal_start_thread
+ *           RtlUserThreadStart (NTDLL.@)
  */
-__ASM_GLOBAL_FUNC( signal_start_thread,
-                   "mov sp, r0\n\t"  /* context */
-                   "mov r1, #1\n\t"
-                   "b " __ASM_NAME("NtContinue") )
+void WINAPI RtlUserThreadStart( PRTL_THREAD_START_ROUTINE entry, void *arg )
+{
+    __TRY
+    {
+        pBaseThreadInitThunk( 0, (LPTHREAD_START_ROUTINE)entry, arg );
+    }
+    __EXCEPT(call_unhandled_exception_filter)
+    {
+        NtTerminateProcess( GetCurrentProcess(), GetExceptionCode() );
+    }
+    __ENDTRY
+}
+
+/******************************************************************
+ *		LdrInitializeThunk (NTDLL.@)
+ */
+void WINAPI LdrInitializeThunk( CONTEXT *context, ULONG_PTR unk2, ULONG_PTR unk3, ULONG_PTR unk4 )
+{
+    loader_init( context, (void **)&context->R0 );
+    TRACE_(relay)( "\1Starting thread proc %p (arg=%p)\n", (void *)context->R0, (void *)context->R1 );
+    NtContinue( context, TRUE );
+}
 
 /**********************************************************************
  *              DbgBreakPoint   (NTDLL.@)
  */
-__ASM_STDCALL_FUNC( DbgBreakPoint, 0, "udf #0xfe; bx lr; nop; nop; nop; nop" );
+__ASM_GLOBAL_FUNC( DbgBreakPoint, "udf #0xfe; bx lr; nop; nop; nop; nop" );
 
 /**********************************************************************
  *              DbgUserBreakPoint   (NTDLL.@)
  */
-__ASM_STDCALL_FUNC( DbgUserBreakPoint, 0, "udf #0xfe; bx lr; nop; nop; nop; nop" );
+__ASM_GLOBAL_FUNC( DbgUserBreakPoint, "udf #0xfe; bx lr; nop; nop; nop; nop" );
 
 #endif  /* __arm__ */
