@@ -59,8 +59,10 @@
 #include <libprocstat.h>
 #endif
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
+#ifdef __APPLE__
+#include <mach/mach.h>
+#endif
+
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #include "winternl.h"
@@ -75,7 +77,7 @@ WINE_DECLARE_DEBUG_CHANNEL(threadname);
 
 pthread_key_t teb_key = 0;
 
-static int nb_threads = 1;
+static LONG nb_threads = 1;
 
 static inline int get_unix_exit_code( NTSTATUS status )
 {
@@ -203,6 +205,25 @@ static unsigned int get_server_context_flags( const void *context, USHORT machin
 }
 
 
+/***********************************************************************
+ *           get_native_context_flags
+ *
+ * Get flags for registers that are set from the native context in WoW mode.
+ */
+static unsigned int get_native_context_flags( USHORT native_machine, USHORT wow_machine )
+{
+    switch (MAKELONG( native_machine, wow_machine ))
+    {
+    case MAKELONG( IMAGE_FILE_MACHINE_AMD64, IMAGE_FILE_MACHINE_I386 ):
+        return SERVER_CTX_DEBUG_REGISTERS | SERVER_CTX_FLOATING_POINT | SERVER_CTX_YMM_REGISTERS;
+    case MAKELONG( IMAGE_FILE_MACHINE_ARM64, IMAGE_FILE_MACHINE_ARMNT ):
+        return SERVER_CTX_DEBUG_REGISTERS | SERVER_CTX_FLOATING_POINT;
+    default:
+        return 0;
+    }
+}
+
+
 /***********************************************************************
  *           context_to_server
  *
@@ -397,7 +418,7 @@ static NTSTATUS context_to_server( context_t *to, USHORT to_machine, const void
         if (flags & CONTEXT_AMD64_FLOATING_POINT)
         {
             to->flags |= SERVER_CTX_FLOATING_POINT;
-            memcpy( to->fp.x86_64_regs.fpregs, &from->u.FltSave, sizeof(to->fp.x86_64_regs.fpregs) );
+            memcpy( to->fp.x86_64_regs.fpregs, &from->FltSave, sizeof(to->fp.x86_64_regs.fpregs) );
         }
         if (flags & CONTEXT_AMD64_DEBUG_REGISTERS)
         {
@@ -458,8 +479,8 @@ static NTSTATUS context_to_server( context_t *to, USHORT to_machine, const void
             I386_FLOATING_SAVE_AREA fpu;
 
             to->flags |= SERVER_CTX_EXTENDED_REGISTERS | SERVER_CTX_FLOATING_POINT;
-            memcpy( to->ext.i386_regs, &from->u.FltSave, sizeof(to->ext.i386_regs) );
-            fpux_to_fpu( &fpu, &from->u.FltSave );
+            memcpy( to->ext.i386_regs, &from->FltSave, sizeof(to->ext.i386_regs) );
+            fpux_to_fpu( &fpu, &from->FltSave );
             to->fp.i386_regs.ctrl     = fpu.ControlWord;
             to->fp.i386_regs.status   = fpu.StatusWord;
             to->fp.i386_regs.tag      = fpu.TagWord;
@@ -524,7 +545,7 @@ static NTSTATUS context_to_server( context_t *to, USHORT to_machine, const void
         if (flags & CONTEXT_ARM_FLOATING_POINT)
         {
             to->flags |= SERVER_CTX_FLOATING_POINT;
-            for (i = 0; i < 32; i++) to->fp.arm_regs.d[i] = from->u.D[i];
+            for (i = 0; i < 32; i++) to->fp.arm_regs.d[i] = from->D[i];
             to->fp.arm_regs.fpscr = from->Fpscr;
         }
         if (flags & CONTEXT_ARM_DEBUG_REGISTERS)
@@ -546,8 +567,8 @@ static NTSTATUS context_to_server( context_t *to, USHORT to_machine, const void
         if (flags & CONTEXT_ARM64_CONTROL)
         {
             to->flags |= SERVER_CTX_CONTROL;
-            to->integer.arm64_regs.x[29] = from->u.s.Fp;
-            to->integer.arm64_regs.x[30] = from->u.s.Lr;
+            to->integer.arm64_regs.x[29] = from->Fp;
+            to->integer.arm64_regs.x[30] = from->Lr;
             to->ctl.arm64_regs.sp     = from->Sp;
             to->ctl.arm64_regs.pc     = from->Pc;
             to->ctl.arm64_regs.pstate = from->Cpsr;
@@ -555,15 +576,15 @@ static NTSTATUS context_to_server( context_t *to, USHORT to_machine, const void
         if (flags & CONTEXT_ARM64_INTEGER)
         {
             to->flags |= SERVER_CTX_INTEGER;
-            for (i = 0; i <= 28; i++) to->integer.arm64_regs.x[i] = from->u.X[i];
+            for (i = 0; i <= 28; i++) to->integer.arm64_regs.x[i] = from->X[i];
         }
         if (flags & CONTEXT_ARM64_FLOATING_POINT)
         {
             to->flags |= SERVER_CTX_FLOATING_POINT;
             for (i = 0; i < 32; i++)
             {
-                to->fp.arm64_regs.q[i].low = from->V[i].s.Low;
-                to->fp.arm64_regs.q[i].high = from->V[i].s.High;
+                to->fp.arm64_regs.q[i].low = from->V[i].Low;
+                to->fp.arm64_regs.q[i].high = from->V[i].High;
             }
             to->fp.arm64_regs.fpcr = from->Fpcr;
             to->fp.arm64_regs.fpsr = from->Fpsr;
@@ -579,6 +600,10 @@ static NTSTATUS context_to_server( context_t *to, USHORT to_machine, const void
         return STATUS_SUCCESS;
     }
 
+    case MAKELONG( IMAGE_FILE_MACHINE_ARM64, IMAGE_FILE_MACHINE_I386 ):
+    case MAKELONG( IMAGE_FILE_MACHINE_I386, IMAGE_FILE_MACHINE_ARM64 ):
+        return STATUS_SUCCESS;
+
     default:
         return STATUS_INVALID_PARAMETER;
     }
@@ -793,8 +818,8 @@ static NTSTATUS context_from_server( void *dst, const context_t *from, USHORT ma
         if ((from->flags & SERVER_CTX_FLOATING_POINT) && (to_flags & CONTEXT_AMD64_FLOATING_POINT))
         {
             to->ContextFlags |= CONTEXT_AMD64_FLOATING_POINT;
-            memcpy( &to->u.FltSave, from->fp.x86_64_regs.fpregs, sizeof(from->fp.x86_64_regs.fpregs) );
-            to->MxCsr = to->u.FltSave.MxCsr;
+            memcpy( &to->FltSave, from->fp.x86_64_regs.fpregs, sizeof(from->fp.x86_64_regs.fpregs) );
+            to->MxCsr = to->FltSave.MxCsr;
         }
         if ((from->flags & SERVER_CTX_DEBUG_REGISTERS) && (to_flags & CONTEXT_AMD64_DEBUG_REGISTERS))
         {
@@ -860,7 +885,7 @@ static NTSTATUS context_from_server( void *dst, const context_t *from, USHORT ma
         if ((from->flags & SERVER_CTX_EXTENDED_REGISTERS) && (to_flags & CONTEXT_AMD64_FLOATING_POINT))
         {
             to->ContextFlags |= CONTEXT_AMD64_FLOATING_POINT;
-            memcpy( &to->u.FltSave, from->ext.i386_regs, sizeof(to->u.FltSave) );
+            memcpy( &to->FltSave, from->ext.i386_regs, sizeof(to->FltSave) );
         }
         else if ((from->flags & SERVER_CTX_FLOATING_POINT) && (to_flags & CONTEXT_AMD64_FLOATING_POINT))
         {
@@ -876,7 +901,7 @@ static NTSTATUS context_from_server( void *dst, const context_t *from, USHORT ma
             fpu.DataSelector  = from->fp.i386_regs.data_sel;
             fpu.Cr0NpxState   = from->fp.i386_regs.cr0npx;
             memcpy( fpu.RegisterArea, from->fp.i386_regs.regs, sizeof(fpu.RegisterArea) );
-            fpu_to_fpux( &to->u.FltSave, &fpu );
+            fpu_to_fpux( &to->FltSave, &fpu );
         }
         if ((from->flags & SERVER_CTX_DEBUG_REGISTERS) && (to_flags & CONTEXT_AMD64_DEBUG_REGISTERS))
         {
@@ -939,7 +964,7 @@ static NTSTATUS context_from_server( void *dst, const context_t *from, USHORT ma
         if ((from->flags & SERVER_CTX_FLOATING_POINT) && (to_flags & CONTEXT_ARM_FLOATING_POINT))
         {
             to->ContextFlags |= CONTEXT_ARM_FLOATING_POINT;
-            for (i = 0; i < 32; i++) to->u.D[i] = from->fp.arm_regs.d[i];
+            for (i = 0; i < 32; i++) to->D[i] = from->fp.arm_regs.d[i];
             to->Fpscr = from->fp.arm_regs.fpscr;
         }
         if ((from->flags & SERVER_CTX_DEBUG_REGISTERS) && (to_flags & CONTEXT_ARM_DEBUG_REGISTERS))
@@ -958,28 +983,27 @@ static NTSTATUS context_from_server( void *dst, const context_t *from, USHORT ma
         ARM64_NT_CONTEXT *to = dst;
 
         to_flags = to->ContextFlags & ~CONTEXT_ARM64;
-        to->ContextFlags = CONTEXT_ARM64;
         if ((from->flags & SERVER_CTX_CONTROL) && (to_flags & CONTEXT_ARM64_CONTROL))
         {
             to->ContextFlags |= CONTEXT_ARM64_CONTROL;
-            to->u.s.Fp = from->integer.arm64_regs.x[29];
-            to->u.s.Lr = from->integer.arm64_regs.x[30];
-            to->Sp     = from->ctl.arm64_regs.sp;
-            to->Pc     = from->ctl.arm64_regs.pc;
-            to->Cpsr   = from->ctl.arm64_regs.pstate;
+            to->Fp   = from->integer.arm64_regs.x[29];
+            to->Lr   = from->integer.arm64_regs.x[30];
+            to->Sp   = from->ctl.arm64_regs.sp;
+            to->Pc   = from->ctl.arm64_regs.pc;
+            to->Cpsr = from->ctl.arm64_regs.pstate;
         }
         if ((from->flags & SERVER_CTX_INTEGER) && (to_flags & CONTEXT_ARM64_INTEGER))
         {
             to->ContextFlags |= CONTEXT_ARM64_INTEGER;
-            for (i = 0; i <= 28; i++) to->u.X[i] = from->integer.arm64_regs.x[i];
+            for (i = 0; i <= 28; i++) to->X[i] = from->integer.arm64_regs.x[i];
         }
         if ((from->flags & SERVER_CTX_FLOATING_POINT) && (to_flags & CONTEXT_ARM64_FLOATING_POINT))
         {
             to->ContextFlags |= CONTEXT_ARM64_FLOATING_POINT;
             for (i = 0; i < 32; i++)
             {
-                to->V[i].s.Low = from->fp.arm64_regs.q[i].low;
-                to->V[i].s.High = from->fp.arm64_regs.q[i].high;
+                to->V[i].Low = from->fp.arm64_regs.q[i].low;
+                to->V[i].High = from->fp.arm64_regs.q[i].high;
             }
             to->Fpcr = from->fp.arm64_regs.fpcr;
             to->Fpsr = from->fp.arm64_regs.fpsr;
@@ -995,6 +1019,10 @@ static NTSTATUS context_from_server( void *dst, const context_t *from, USHORT ma
         return STATUS_SUCCESS;
     }
 
+    case MAKELONG( IMAGE_FILE_MACHINE_ARM64, IMAGE_FILE_MACHINE_I386 ):
+    case MAKELONG( IMAGE_FILE_MACHINE_I386, IMAGE_FILE_MACHINE_ARM64 ):
+        return STATUS_SUCCESS;
+
     default:
         return STATUS_INVALID_PARAMETER;
     }
@@ -1038,7 +1066,10 @@ static void contexts_from_server( CONTEXT *context, context_t server_contexts[2]
     if (native_context)
     {
         context_from_server( native_context, &server_contexts[0], native_machine );
-        if (wow_context) context_from_server( wow_context, &server_contexts[1], main_image_info.Machine );
+        if (wow_context)
+            context_from_server( wow_context, &server_contexts[1], main_image_info.Machine );
+        else
+            context_from_server( native_context, &server_contexts[1], native_machine );
     }
     else context_from_server( wow_context, &server_contexts[0], main_image_info.Machine );
 }
@@ -1100,7 +1131,7 @@ void *get_cpu_area( USHORT machine )
     WOW64_CPURESERVED *cpu;
     ULONG align;
 
-    if (!NtCurrentTeb()->WowTebOffset) return NULL;
+    if (!is_wow64()) return NULL;
 #ifdef _WIN64
     cpu = NtCurrentTeb()->TlsSlots[WOW64_TLS_CPURESERVED];
 #else
@@ -1141,44 +1172,43 @@ void set_thread_id( TEB *teb, DWORD pid, DWORD tid )
 /***********************************************************************
  *           init_thread_stack
  */
-NTSTATUS init_thread_stack( TEB *teb, ULONG_PTR zero_bits, SIZE_T reserve_size, SIZE_T commit_size )
+NTSTATUS init_thread_stack( TEB *teb, ULONG_PTR limit, SIZE_T reserve_size, SIZE_T commit_size )
 {
     struct ntdll_thread_data *thread_data = (struct ntdll_thread_data *)&teb->GdiTebBatch;
     WOW_TEB *wow_teb = get_wow_teb( teb );
     INITIAL_TEB stack;
     NTSTATUS status;
 
+    /* kernel stack */
+    if ((status = virtual_alloc_thread_stack( &stack, limit_4g, 0, kernel_stack_size, kernel_stack_size, FALSE )))
+        return status;
+    thread_data->kernel_stack = stack.DeallocationStack;
+
     if (wow_teb)
     {
         WOW64_CPURESERVED *cpu;
         SIZE_T cpusize = sizeof(WOW64_CPURESERVED) +
             ((get_machine_context_size( main_image_info.Machine ) + 7) & ~7) + sizeof(ULONG64);
 
+        /* 64-bit stack */
+        if ((status = virtual_alloc_thread_stack( &stack, limit_4g, 0, 0x40000, 0x40000, TRUE ))) return status;
+        cpu = (WOW64_CPURESERVED *)(((ULONG_PTR)stack.StackBase - cpusize) & ~15);
+        cpu->Machine = main_image_info.Machine;
+
 #ifdef _WIN64
+        teb->Tib.StackBase = teb->TlsSlots[WOW64_TLS_CPURESERVED] = cpu;
+        teb->Tib.StackLimit = stack.StackLimit;
+        teb->DeallocationStack = stack.DeallocationStack;
+
         /* 32-bit stack */
-        if ((status = virtual_alloc_thread_stack( &stack, zero_bits ? zero_bits : 0x7fffffff,
-                                                  reserve_size, commit_size, 0 )))
+        if (!limit || limit > user_space_wow_limit) limit = user_space_wow_limit;
+        if ((status = virtual_alloc_thread_stack( &stack, 0, limit, reserve_size, commit_size, TRUE )))
             return status;
         wow_teb->Tib.StackBase = PtrToUlong( stack.StackBase );
         wow_teb->Tib.StackLimit = PtrToUlong( stack.StackLimit );
         wow_teb->DeallocationStack = PtrToUlong( stack.DeallocationStack );
-
-        /* 64-bit stack */
-        if ((status = virtual_alloc_thread_stack( &stack, 0, 0x40000, 0x40000, kernel_stack_size )))
-            return status;
-        cpu = (WOW64_CPURESERVED *)(((ULONG_PTR)stack.StackBase - cpusize) & ~15);
-        cpu->Machine = main_image_info.Machine;
-        teb->Tib.StackBase = teb->TlsSlots[WOW64_TLS_CPURESERVED] = cpu;
-        teb->Tib.StackLimit = stack.StackLimit;
-        teb->DeallocationStack = stack.DeallocationStack;
-        thread_data->kernel_stack = stack.StackBase;
         return STATUS_SUCCESS;
 #else
-        /* 64-bit stack */
-        if ((status = virtual_alloc_thread_stack( &stack, 0, 0x40000, 0x40000, 0 ))) return status;
-
-        cpu = (WOW64_CPURESERVED *)(((ULONG_PTR)stack.StackBase - cpusize) & ~15);
-        cpu->Machine = main_image_info.Machine;
         wow_teb->Tib.StackBase = wow_teb->TlsSlots[WOW64_TLS_CPURESERVED] = PtrToUlong( cpu );
         wow_teb->Tib.StackLimit = PtrToUlong( stack.StackLimit );
         wow_teb->DeallocationStack = PtrToUlong( stack.DeallocationStack );
@@ -1186,13 +1216,11 @@ NTSTATUS init_thread_stack( TEB *teb, ULONG_PTR zero_bits, SIZE_T reserve_size,
     }
 
     /* native stack */
-    if ((status = virtual_alloc_thread_stack( &stack, zero_bits, reserve_size,
-                                              commit_size, kernel_stack_size )))
+    if ((status = virtual_alloc_thread_stack( &stack, 0, limit, reserve_size, commit_size, TRUE )))
         return status;
     teb->Tib.StackBase = stack.StackBase;
     teb->Tib.StackLimit = stack.StackLimit;
     teb->DeallocationStack = stack.DeallocationStack;
-    thread_data->kernel_stack = stack.StackBase;
     return STATUS_SUCCESS;
 }
 
@@ -1230,7 +1258,8 @@ NTSTATUS WINAPI NtCreateThread( HANDLE *handle, ACCESS_MASK access, OBJECT_ATTRI
                                 HANDLE process, CLIENT_ID *id, CONTEXT *ctx, INITIAL_TEB *teb,
                                 BOOLEAN suspended )
 {
-    FIXME( "%p %d %p %p %p %p %p %d, stub!\n", handle, access, attr, process, id, ctx, teb, suspended );
+    FIXME( "%p %d %p %p %p %p %p %d, stub!\n",
+           handle, (int)access, attr, process, id, ctx, teb, suspended );
     return STATUS_NOT_IMPLEMENTED;
 }
 
@@ -1252,14 +1281,14 @@ NTSTATUS WINAPI NtCreateThreadEx( HANDLE *handle, ACCESS_MASK access, OBJECT_ATT
     DWORD tid = 0;
     int request_pipe[2];
     TEB *teb;
-    NTSTATUS status;
+    unsigned int status;
 
     if (flags & ~supported_flags)
-        FIXME( "Unsupported flags %#x.\n", flags );
+        FIXME( "Unsupported flags %#x.\n", (int)flags );
 
     if (zero_bits > 21 && zero_bits < 32) return STATUS_INVALID_PARAMETER_3;
 #ifndef _WIN64
-    if (!is_wow64 && zero_bits >= 32) return STATUS_INVALID_PARAMETER_3;
+    if (!is_old_wow64() && zero_bits >= 32) return STATUS_INVALID_PARAMETER_3;
 #endif
 
     if (process != NtCurrentProcess())
@@ -1329,7 +1358,7 @@ NTSTATUS WINAPI NtCreateThreadEx( HANDLE *handle, ACCESS_MASK access, OBJECT_ATT
 
     if ((status = virtual_alloc_teb( &teb ))) goto done;
 
-    if ((status = init_thread_stack( teb, zero_bits, stack_reserve, stack_commit )))
+    if ((status = init_thread_stack( teb, get_zero_bits_limit( zero_bits ), stack_reserve, stack_commit )))
     {
         virtual_free_teb( teb );
         goto done;
@@ -1343,8 +1372,7 @@ NTSTATUS WINAPI NtCreateThreadEx( HANDLE *handle, ACCESS_MASK access, OBJECT_ATT
     thread_data->param = param;
 
     pthread_attr_init( &pthread_attr );
-    pthread_attr_setstack( &pthread_attr, teb->DeallocationStack,
-                           (char *)thread_data->kernel_stack + kernel_stack_size - (char *)teb->DeallocationStack );
+    pthread_attr_setstack( &pthread_attr, thread_data->kernel_stack, kernel_stack_size );
     pthread_attr_setguardsize( &pthread_attr, 0 );
     pthread_attr_setscope( &pthread_attr, PTHREAD_SCOPE_SYSTEM ); /* force creating a kernel thread */
     InterlockedIncrement( &nb_threads );
@@ -1399,6 +1427,8 @@ static DECLSPEC_NORETURN void exit_thread( int status )
 
     pthread_sigmask( SIG_BLOCK, &server_block_set, NULL );
 
+    if (InterlockedDecrement( &nb_threads ) <= 0) exit_process( status );
+
     if ((teb = InterlockedExchangePointer( &prev_teb, NtCurrentTeb() )))
     {
         struct ntdll_thread_data *thread_data = (struct ntdll_thread_data *)&teb->GdiTebBatch;
@@ -1448,7 +1478,7 @@ void wait_suspend( CONTEXT *context )
  */
 NTSTATUS send_debug_event( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL first_chance )
 {
-    NTSTATUS ret;
+    unsigned int ret;
     DWORD i;
     obj_handle_t handle = 0;
     client_ptr_t params[EXCEPTION_MAXIMUM_PARAMETERS];
@@ -1492,7 +1522,7 @@ NTSTATUS send_debug_event( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL first_c
             ret = wine_server_call( req );
         }
         SERVER_END_REQ;
-        if (ret >= 0) contexts_from_server( context, server_contexts );
+        if (NT_SUCCESS(ret)) contexts_from_server( context, server_contexts );
     }
 
     pthread_sigmask( SIG_SETMASK, &old_set, NULL );
@@ -1518,7 +1548,7 @@ NTSTATUS WINAPI NtRaiseException( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL
         ERR_(seh)("Process attempted to continue execution after noncontinuable exception.\n");
     else
         ERR_(seh)("Unhandled exception code %x flags %x addr %p\n",
-                  rec->ExceptionCode, rec->ExceptionFlags, rec->ExceptionAddress );
+                  (int)rec->ExceptionCode, (int)rec->ExceptionFlags, rec->ExceptionAddress );
 
     NtTerminateProcess( NtCurrentProcess(), rec->ExceptionCode );
     return STATUS_SUCCESS;
@@ -1540,7 +1570,7 @@ TEB * WINAPI NtCurrentTeb(void)
 NTSTATUS WINAPI NtOpenThread( HANDLE *handle, ACCESS_MASK access,
                               const OBJECT_ATTRIBUTES *attr, const CLIENT_ID *id )
 {
-    NTSTATUS ret;
+    unsigned int ret;
 
     *handle = 0;
 
@@ -1562,7 +1592,7 @@ NTSTATUS WINAPI NtOpenThread( HANDLE *handle, ACCESS_MASK access,
  */
 NTSTATUS WINAPI NtSuspendThread( HANDLE handle, ULONG *count )
 {
-    NTSTATUS ret;
+    unsigned int ret;
 
     SERVER_START_REQ( suspend_thread )
     {
@@ -1582,7 +1612,7 @@ NTSTATUS WINAPI NtSuspendThread( HANDLE handle, ULONG *count )
  */
 NTSTATUS WINAPI NtResumeThread( HANDLE handle, ULONG *count )
 {
-    NTSTATUS ret;
+    unsigned int ret;
 
     SERVER_START_REQ( resume_thread )
     {
@@ -1622,7 +1652,7 @@ NTSTATUS WINAPI NtAlertThread( HANDLE handle )
  */
 NTSTATUS WINAPI NtTerminateThread( HANDLE handle, LONG exit_code )
 {
-    NTSTATUS ret;
+    unsigned int ret;
     BOOL self;
 
     SERVER_START_REQ( terminate_thread )
@@ -1649,20 +1679,21 @@ NTSTATUS WINAPI NtTerminateThread( HANDLE handle, LONG exit_code )
 NTSTATUS WINAPI NtQueueApcThread( HANDLE handle, PNTAPCFUNC func, ULONG_PTR arg1,
                                   ULONG_PTR arg2, ULONG_PTR arg3 )
 {
-    NTSTATUS ret;
+    unsigned int ret;
+    apc_call_t call;
 
     SERVER_START_REQ( queue_apc )
     {
         req->handle = wine_server_obj_handle( handle );
         if (func)
         {
-            req->call.type         = APC_USER;
-            req->call.user.func    = wine_server_client_ptr( func );
-            req->call.user.args[0] = arg1;
-            req->call.user.args[1] = arg2;
-            req->call.user.args[2] = arg3;
+            call.type         = APC_USER;
+            call.user.func    = wine_server_client_ptr( func );
+            call.user.args[0] = arg1;
+            call.user.args[1] = arg2;
+            call.user.args[2] = arg3;
+            wine_server_add_data( req, &call, sizeof(call) );
         }
-        else req->call.type = APC_NONE;  /* wake up only */
         ret = wine_server_call( req );
     }
     SERVER_END_REQ;
@@ -1677,7 +1708,7 @@ NTSTATUS set_thread_context( HANDLE handle, const void *context, BOOL *self, USH
 {
     context_t server_contexts[2];
     unsigned int count = 0;
-    NTSTATUS ret;
+    unsigned int ret;
 
     context_to_server( &server_contexts[count++], native_machine, context, machine );
     if (machine != native_machine)
@@ -1686,6 +1717,7 @@ NTSTATUS set_thread_context( HANDLE handle, const void *context, BOOL *self, USH
     SERVER_START_REQ( set_thread_context )
     {
         req->handle  = wine_server_obj_handle( handle );
+        req->native_flags = server_contexts[0].flags & get_native_context_flags( native_machine, machine );
         wine_server_add_data( req, server_contexts, count * sizeof(server_contexts[0]) );
         ret = wine_server_call( req );
         *self = reply->self;
@@ -1701,7 +1733,7 @@ NTSTATUS set_thread_context( HANDLE handle, const void *context, BOOL *self, USH
  */
 NTSTATUS get_thread_context( HANDLE handle, void *context, BOOL *self, USHORT machine )
 {
-    NTSTATUS ret;
+    unsigned int ret;
     HANDLE context_handle;
     context_t server_contexts[2];
     unsigned int count;
@@ -1712,6 +1744,7 @@ NTSTATUS get_thread_context( HANDLE handle, void *context, BOOL *self, USHORT ma
         req->handle  = wine_server_obj_handle( handle );
         req->flags   = flags;
         req->machine = machine;
+        req->native_flags = flags & get_native_context_flags( native_machine, machine );
         wine_server_set_reply( req, server_contexts, sizeof(server_contexts) );
         ret = wine_server_call( req );
         *self = reply->self;
@@ -1729,6 +1762,7 @@ NTSTATUS get_thread_context( HANDLE handle, void *context, BOOL *self, USHORT ma
             req->context = wine_server_obj_handle( context_handle );
             req->flags   = flags;
             req->machine = machine;
+            req->native_flags = flags & get_native_context_flags( native_machine, machine );
             wine_server_set_reply( req, server_contexts, sizeof(server_contexts) );
             ret = wine_server_call( req );
             count = wine_server_reply_size( reply ) / sizeof(server_contexts[0]);
@@ -1765,9 +1799,9 @@ BOOL get_thread_times(int unix_pid, int unix_tid, LARGE_INTEGER *kernel_time, LA
     int i;
 
     if (unix_tid == -1)
-        sprintf( buf, "/proc/%u/stat", unix_pid );
+        snprintf( buf, sizeof(buf), "/proc/%u/stat", unix_pid );
     else
-        sprintf( buf, "/proc/%u/task/%u/stat", unix_pid, unix_tid );
+        snprintf( buf, sizeof(buf), "/proc/%u/task/%u/stat", unix_pid, unix_tid );
     if (!(f = fopen( buf, "r" )))
     {
         WARN("Failed to open %s: %s\n", buf, strerror(errno));
@@ -1843,7 +1877,7 @@ BOOL get_thread_times(int unix_pid, int unix_tid, LARGE_INTEGER *kernel_time, LA
 static void set_native_thread_name( HANDLE handle, const UNICODE_STRING *name )
 {
 #ifdef linux
-    NTSTATUS status;
+    unsigned int status;
     char path[64], nameA[64];
     int unix_pid, unix_tid, len, fd;
 
@@ -1870,26 +1904,59 @@ static void set_native_thread_name( HANDLE handle, const UNICODE_STRING *name )
     }
 
     len = ntdll_wcstoumbs( name->Buffer, name->Length / sizeof(WCHAR), nameA, sizeof(nameA), FALSE );
-    sprintf(path, "/proc/%u/task/%u/comm", unix_pid, unix_tid);
+    snprintf(path, sizeof(path), "/proc/%u/task/%u/comm", unix_pid, unix_tid);
     if ((fd = open( path, O_WRONLY )) != -1)
     {
         write( fd, nameA, len );
         close( fd );
     }
+#elif defined(__APPLE__)
+    /* pthread_setname_np() silently fails if the name is longer than 63 characters + null terminator */
+    char nameA[64];
+    NTSTATUS status;
+    int unix_pid, unix_tid, len, current_tid;
+
+    SERVER_START_REQ( get_thread_times )
+    {
+        req->handle = wine_server_obj_handle( handle );
+        status = wine_server_call( req );
+        if (status == STATUS_SUCCESS)
+        {
+            unix_pid = reply->unix_pid;
+            unix_tid = reply->unix_tid;
+        }
+    }
+    SERVER_END_REQ;
+
+    if (status != STATUS_SUCCESS || unix_pid == -1 || unix_tid == -1)
+        return;
+
+    current_tid = mach_thread_self();
+    mach_port_deallocate(mach_task_self(), current_tid);
+
+    if (unix_tid != current_tid)
+    {
+        static int once;
+        if (!once++) FIXME("setting other thread name not supported\n");
+        return;
+    }
+
+    len = ntdll_wcstoumbs( name->Buffer, name->Length / sizeof(WCHAR), nameA, sizeof(nameA) - 1, FALSE );
+    nameA[len] = '\0';
+    pthread_setname_np(nameA);
 #else
     static int once;
     if (!once++) FIXME("not implemented on this platform\n");
 #endif
 }
 
-#ifndef _WIN64
 static BOOL is_process_wow64( const CLIENT_ID *id )
 {
     HANDLE handle;
     ULONG_PTR info;
     BOOL ret = FALSE;
 
-    if (id->UniqueProcess == ULongToHandle(GetCurrentProcessId())) return is_wow64;
+    if (id->UniqueProcess == ULongToHandle(GetCurrentProcessId())) return is_old_wow64();
     if (!NtOpenProcess( &handle, PROCESS_QUERY_LIMITED_INFORMATION, NULL, id ))
     {
         if (!NtQueryInformationProcess( handle, ProcessWow64Information, &info, sizeof(info), NULL ))
@@ -1898,7 +1965,6 @@ static BOOL is_process_wow64( const CLIENT_ID *id )
     }
     return ret;
 }
-#endif
 
 /******************************************************************************
  *              NtQueryInformationThread  (NTDLL.@)
@@ -1906,9 +1972,9 @@ static BOOL is_process_wow64( const CLIENT_ID *id )
 NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
                                           void *data, ULONG length, ULONG *ret_len )
 {
-    NTSTATUS status;
+    unsigned int status;
 
-    TRACE("(%p,%d,%p,%x,%p)\n", handle, class, data, length, ret_len);
+    TRACE("(%p,%d,%p,%x,%p)\n", handle, class, data, (int)length, ret_len);
 
     switch (class)
     {
@@ -1934,15 +2000,13 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
         SERVER_END_REQ;
         if (status == STATUS_SUCCESS)
         {
-#ifndef _WIN64
-            if (is_wow64)
+            if (is_old_wow64())
             {
                 if (is_process_wow64( &info.ClientId ))
                     info.TebBaseAddress = (char *)info.TebBaseAddress + teb_offset;
                 else
                     info.TebBaseAddress = NULL;
             }
-#endif
             if (data) memcpy( data, &info, min( length, sizeof(info) ));
             if (ret_len) *ret_len = min( length, sizeof(info) );
         }
@@ -2138,6 +2202,17 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
         if (ret_len) *ret_len = sizeof(BOOLEAN);
         return STATUS_SUCCESS;
 
+    case ThreadPriorityBoost:
+    {
+        DWORD *value = data;
+
+        if (length != sizeof(ULONG)) return STATUS_INFO_LENGTH_MISMATCH;
+        if (ret_len) *ret_len = sizeof(ULONG);
+        *value = 0;
+        return STATUS_SUCCESS;
+    }
+
+    case ThreadIdealProcessor:
     case ThreadEnableAlignmentFaultFixup:
         return STATUS_INVALID_INFO_CLASS;
 
@@ -2147,8 +2222,6 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
     case ThreadEventPair_Reusable:
     case ThreadZeroTlsCell:
     case ThreadPerformanceCount:
-    case ThreadIdealProcessor:
-    case ThreadPriorityBoost:
     case ThreadSetTlsArrayAddress:
     default:
         FIXME( "info class %d not supported yet\n", class );
@@ -2163,9 +2236,9 @@ NTSTATUS WINAPI NtQueryInformationThread( HANDLE handle, THREADINFOCLASS class,
 NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
                                         const void *data, ULONG length )
 {
-    NTSTATUS status;
+    unsigned int status;
 
-    TRACE("(%p,%d,%p,%x)\n", handle, class, data, length);
+    TRACE("(%p,%d,%p,%x)\n", handle, class, data, (int)length);
 
     switch (class)
     {
@@ -2297,7 +2370,7 @@ NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
         if (handle == GetCurrentThread() || (!status && (HandleToULong(tbi.ClientId.UniqueThread) == GetCurrentThreadId())))
             WARN_(threadname)( "Thread renamed to %s\n", debugstr_us(&info->ThreadName) );
         else if (!status)
-            WARN_(threadname)( "Thread ID %04x renamed to %s\n", HandleToULong( tbi.ClientId.UniqueThread ), debugstr_us(&info->ThreadName) );
+            WARN_(threadname)( "Thread ID %04x renamed to %s\n", (int)HandleToULong( tbi.ClientId.UniqueThread ), debugstr_us(&info->ThreadName) );
         else
             WARN_(threadname)( "Thread handle %p renamed to %s\n", handle, debugstr_us(&info->ThreadName) );
 
@@ -2340,6 +2413,20 @@ NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
         FIXME( "ThreadPowerThrottling stub!\n" );
         return STATUS_SUCCESS;
 
+    case ThreadIdealProcessor:
+    {
+        const ULONG *number = data;
+
+        if (length != sizeof(*number)) return STATUS_INFO_LENGTH_MISMATCH;
+        if (*number > MAXIMUM_PROCESSORS) return STATUS_INVALID_PARAMETER;
+        FIXME( "ThreadIdealProcessor stub!\n" );
+        return STATUS_SUCCESS;
+    }
+
+    case ThreadPriorityBoost:
+        WARN("Unimplemented class ThreadPriorityBoost.\n");
+        return STATUS_SUCCESS;
+
     case ThreadBasicInformation:
     case ThreadTimes:
     case ThreadPriority:
@@ -2347,8 +2434,6 @@ NTSTATUS WINAPI NtSetInformationThread( HANDLE handle, THREADINFOCLASS class,
     case ThreadEventPair_Reusable:
     case ThreadPerformanceCount:
     case ThreadAmILastThread:
-    case ThreadIdealProcessor:
-    case ThreadPriorityBoost:
     case ThreadSetTlsArrayAddress:
     case ThreadIsIoPending:
     default:
@@ -2384,7 +2469,7 @@ ULONG WINAPI NtGetCurrentProcessorNumber(void)
                 {
                     if (thread_mask != processor_mask)
                         FIXME( "need multicore support (%d processors)\n",
-                               peb->NumberOfProcessors );
+                               (int)peb->NumberOfProcessors );
                     return processor;
                 }
             }
@@ -2402,10 +2487,10 @@ NTSTATUS WINAPI NtGetNextThread( HANDLE process, HANDLE thread, ACCESS_MASK acce
                                  ULONG flags, HANDLE *handle )
 {
     HANDLE ret_handle = 0;
-    NTSTATUS ret;
+    unsigned int ret;
 
     TRACE( "process %p, thread %p, access %#x, attributes %#x, flags %#x, handle %p.\n",
-            process, thread, access, attributes, flags, handle );
+            process, thread, (int)access, (int)attributes, (int)flags, handle );
 
     SERVER_START_REQ( get_next_thread )
     {
