@@ -24,10 +24,10 @@
 #include <sys/types.h>
 #include <limits.h>
 #include "schannel.h"
+#include "ntsecapi.h"
+#include "ntsecpkg.h"
 #include "wine/list.h"
 
-extern HINSTANCE hsecur32 DECLSPEC_HIDDEN;
-
 typedef struct _SecureProvider
 {
     struct list             entry;
@@ -72,8 +72,10 @@ SecurePackage *SECUR32_findPackageA(PCSTR packageName) DECLSPEC_HIDDEN;
 
 /* Initialization functions for built-in providers */
 void SECUR32_initSchannelSP(void) DECLSPEC_HIDDEN;
-void SECUR32_initNegotiateSP(void) DECLSPEC_HIDDEN;
 void load_auth_packages(void) DECLSPEC_HIDDEN;
+NTSTATUS NTAPI nego_SpLsaModeInitialize(ULONG, PULONG, PSECPKG_FUNCTION_TABLE *, PULONG) DECLSPEC_HIDDEN;
+NTSTATUS NTAPI nego_SpUserModeInitialize(ULONG, PULONG, PSECPKG_USER_FUNCTION_TABLE *, PULONG) DECLSPEC_HIDDEN;
+SECPKG_FUNCTION_TABLE *lsa_find_package(const char *name, SECPKG_USER_FUNCTION_TABLE **user_api) DECLSPEC_HIDDEN;
 
 /* Cleanup functions for built-in providers */
 void SECUR32_deinitSchannelSP(void) DECLSPEC_HIDDEN;
@@ -149,8 +151,9 @@ struct get_unique_channel_binding_params
 
 enum control_token
 {
-    control_token_none,
-    control_token_shutdown,
+    CONTROL_TOKEN_NONE,
+    CONTROL_TOKEN_SHUTDOWN,
+    CONTROL_TOKEN_ALERT,
 };
 
 struct handshake_params
@@ -163,6 +166,8 @@ struct handshake_params
     int *output_buffer_idx;
     ULONG *output_offset;
     enum control_token control_token;
+    unsigned int alert_type;
+    unsigned int alert_number;
 };
 
 struct recv_params
