@@ -586,11 +586,10 @@ static LRESULT dispatch_send_message( struct win_proc_params *params, WPARAM wpa
 
     thread_info->recursion_count++;
 
-    params->result = &retval;
     thread_info->msg_source = msg_source_unavailable;
     SPY_EnterMessage( SPY_SENDMESSAGE, params->hwnd, params->msg, params->wparam, params->lparam );
 
-    dispatch_win_proc_params( params );
+    retval = dispatch_win_proc_params( params );
 
     SPY_ExitMessage( SPY_RESULT_OK, params->hwnd, params->msg, retval, params->wparam, params->lparam );
     thread_info->msg_source = prev_source;
@@ -808,6 +807,14 @@ BOOL WINAPI DECLSPEC_HOTPATCH GetMessageA( MSG *msg, HWND hwnd, UINT first, UINT
     return (msg->message != WM_QUIT);
 }
 
+static BOOL is_cjk(void)
+{
+    int lang_id = PRIMARYLANGID(GetUserDefaultLangID());
+
+    if (lang_id == LANG_CHINESE || lang_id == LANG_JAPANESE || lang_id == LANG_KOREAN)
+        return TRUE;
+    return FALSE;
+}
 
 /***********************************************************************
  *		IsDialogMessageA (USER32.@)
@@ -815,8 +822,12 @@ BOOL WINAPI DECLSPEC_HOTPATCH GetMessageA( MSG *msg, HWND hwnd, UINT first, UINT
  */
 BOOL WINAPI IsDialogMessageA( HWND hwndDlg, LPMSG pmsg )
 {
+    enum wm_char_mapping mapping;
     MSG msg = *pmsg;
-    map_wparam_AtoW( msg.message, &msg.wParam, WMCHAR_MAP_NOMAPPING );
+
+    mapping = is_cjk() ? WMCHAR_MAP_ISDIALOGMESSAGE : WMCHAR_MAP_NOMAPPING;
+    if (!map_wparam_AtoW( msg.message, &msg.wParam, mapping ))
+        return TRUE;
     return IsDialogMessageW( hwndDlg, &msg );
 }
 
@@ -850,10 +861,9 @@ static LRESULT dispatch_message( const MSG *msg, BOOL ansi )
 
     if (!NtUserMessageCall( msg->hwnd, msg->message, msg->wParam, msg->lParam,
                             &params, NtUserGetDispatchParams, ansi )) return 0;
-    params.result = &retval;
 
     SPY_EnterMessage( SPY_DISPATCHMESSAGE, msg->hwnd, msg->message, msg->wParam, msg->lParam );
-    dispatch_win_proc_params( &params );
+    retval = dispatch_win_proc_params( &params );
     SPY_ExitMessage( SPY_RESULT_OK, msg->hwnd, msg->message, retval, msg->wParam, msg->lParam );
     return retval;
 }
@@ -1026,19 +1036,6 @@ BOOL WINAPI GetCurrentInputMessageSource( INPUT_MESSAGE_SOURCE *source )
 }
 
 
-/***********************************************************************
- *		WaitMessage (USER.112) Suspend thread pending messages
- *		WaitMessage (USER32.@) Suspend thread pending messages
- *
- * WaitMessage() suspends a thread until events appear in the thread's
- * queue.
- */
-BOOL WINAPI WaitMessage(void)
-{
-    return NtUserMsgWaitForMultipleObjectsEx( 0, NULL, INFINITE, QS_ALLINPUT, 0 ) != WAIT_FAILED;
-}
-
-
 /***********************************************************************
  *		MsgWaitForMultipleObjects (USER32.@)
  */
@@ -1205,7 +1202,7 @@ LONG WINAPI BroadcastSystemMessageW( DWORD flags, LPDWORD recipients, UINT msg,
 LONG WINAPI BroadcastSystemMessageExA( DWORD flags, LPDWORD recipients, UINT msg, WPARAM wp, LPARAM lp, PBSMINFO pinfo )
 {
     map_wparam_AtoW( msg, &wp, WMCHAR_MAP_NOMAPPING );
-    return BroadcastSystemMessageExW( flags, recipients, msg, wp, lp, NULL );
+    return BroadcastSystemMessageExW( flags, recipients, msg, wp, lp, pinfo );
 }
 
 
