@@ -23,6 +23,7 @@
 #include <stdio.h>
 
 #include "wined3d_private.h"
+#include "wined3d_gl.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d);
 
@@ -88,7 +89,7 @@ struct tex_op_args
     GLenum component_usage[3];
 };
 
-static GLenum d3dta_to_combiner_input(DWORD d3dta, DWORD stage, INT texture_idx) {
+static GLenum d3dta_to_combiner_input(unsigned int d3dta, DWORD stage, INT texture_idx) {
     switch (d3dta) {
         case WINED3DTA_DIFFUSE:
             return GL_PRIMARY_COLOR_NV;
@@ -144,7 +145,7 @@ static void get_src_and_opr_nvrc(DWORD stage, DWORD arg, BOOL is_alpha, GLenum*
 }
 
 void set_tex_op_nvrc(const struct wined3d_gl_info *gl_info, const struct wined3d_state *state, BOOL is_alpha,
-        int stage, enum wined3d_texture_op op, DWORD arg1, DWORD arg2, DWORD arg3, INT texture_idx, DWORD dst)
+        int stage, enum wined3d_texture_op op, uint32_t arg1, uint32_t arg2, uint32_t arg3, INT texture_idx, DWORD dst)
 {
     struct tex_op_args tex_op_args = {{0}, {0}, {0}};
     GLenum portion = is_alpha ? GL_ALPHA : GL_RGB;
@@ -480,7 +481,7 @@ void set_tex_op_nvrc(const struct wined3d_gl_info *gl_info, const struct wined3d
 
 static void nvrc_colorop(struct wined3d_context *context, const struct wined3d_state *state, DWORD state_id)
 {
-    DWORD stage = (state_id - STATE_TEXTURESTAGE(0, 0)) / (WINED3D_HIGHEST_TEXTURE_STATE + 1);
+    unsigned int stage = (state_id - STATE_TEXTURESTAGE(0, 0)) / (WINED3D_HIGHEST_TEXTURE_STATE + 1);
     struct wined3d_context_gl *context_gl = wined3d_context_gl(context);
     BOOL tex_used = context->fixed_function_usage_map & (1u << stage);
     const struct wined3d_gl_info *gl_info = context_gl->gl_info;
@@ -495,7 +496,7 @@ static void nvrc_colorop(struct wined3d_context *context, const struct wined3d_s
 
     if (mapped_stage != WINED3D_UNMAPPED_STAGE)
     {
-        if (tex_used && mapped_stage >= gl_info->limits.textures)
+        if (tex_used && mapped_stage >= gl_info->limits.ffp_textures)
         {
             FIXME("Attempt to enable unsupported stage!\n");
             return;
@@ -532,7 +533,7 @@ static void nvrc_colorop(struct wined3d_context *context, const struct wined3d_s
                 gl_info->gl_ops.gl.p_glDisable(GL_TEXTURE_RECTANGLE_ARB);
                 checkGLcall("glDisable(GL_TEXTURE_RECTANGLE_ARB)");
             }
-            if (gl_info->supported[NV_TEXTURE_SHADER2] && mapped_stage < gl_info->limits.textures)
+            if (gl_info->supported[NV_TEXTURE_SHADER2] && mapped_stage < gl_info->limits.ffp_textures)
             {
                 gl_info->gl_ops.gl.p_glTexEnvi(GL_TEXTURE_SHADER_NV, GL_SHADER_OPERATION_NV, GL_NONE);
             }
@@ -583,7 +584,7 @@ static void nvrc_colorop(struct wined3d_context *context, const struct wined3d_s
 
 static void nvrc_resultarg(struct wined3d_context *context, const struct wined3d_state *state, DWORD state_id)
 {
-    DWORD stage = (state_id - STATE_TEXTURESTAGE(0, 0)) / (WINED3D_HIGHEST_TEXTURE_STATE + 1);
+    unsigned int stage = (state_id - STATE_TEXTURESTAGE(0, 0)) / (WINED3D_HIGHEST_TEXTURE_STATE + 1);
 
     TRACE("Setting result arg for stage %u.\n", stage);
 
@@ -608,7 +609,7 @@ static void nvts_texdim(struct wined3d_context *context, const struct wined3d_st
     /* No need to enable / disable anything here for unused samplers. The tex_colorop
     * handler takes care. Also no action is needed with pixel shaders, or if tex_colorop
     * will take care of this business. */
-    if (mapped_stage == WINED3D_UNMAPPED_STAGE || mapped_stage >= context_gl->gl_info->limits.textures)
+    if (mapped_stage == WINED3D_UNMAPPED_STAGE || mapped_stage >= context_gl->gl_info->limits.ffp_textures)
         return;
     if (sampler >= context->lowest_disabled_stage)
         return;
@@ -632,7 +633,7 @@ static void nvts_bumpenvmat(struct wined3d_context *context, const struct wined3
      * map is read from a specified source stage(always stage - 1 for d3d). Thus set the matrix
      * for stage + 1. Keep the nvrc tex unit mapping in mind too
      */
-    if (mapped_stage < gl_info->limits.textures)
+    if (mapped_stage < gl_info->limits.ffp_textures)
     {
         wined3d_context_gl_active_texture(context_gl, gl_info, mapped_stage);
 
@@ -694,9 +695,10 @@ static void nvts_enable(const struct wined3d_context *context, BOOL enable)
 
 static void nvrc_fragment_get_caps(const struct wined3d_adapter *adapter, struct fragment_caps *caps)
 {
-    const struct wined3d_gl_info *gl_info = &adapter->gl_info;
+    const struct wined3d_gl_info *gl_info = &wined3d_adapter_gl_const(adapter)->gl_info;
+
+    memset(caps, 0, sizeof(*caps));
 
-    caps->wined3d_caps = 0;
     caps->PrimitiveMiscCaps = WINED3DPMISCCAPS_TSSARGTEMP;
 
     /* The caps below can be supported but aren't handled yet in utils.c
@@ -746,11 +748,11 @@ static void nvrc_fragment_get_caps(const struct wined3d_adapter *adapter, struct
             WINED3DTEXOPCAPS_PREMODULATE */
 #endif
 
-    caps->MaxTextureBlendStages = min(WINED3D_MAX_TEXTURES, gl_info->limits.general_combiners);
-    caps->MaxSimultaneousTextures = gl_info->limits.textures;
+    caps->max_blend_stages = min(WINED3D_MAX_FFP_TEXTURES, gl_info->limits.general_combiners);
+    caps->max_textures = gl_info->limits.ffp_textures;
 }
 
-static unsigned int nvrc_fragment_get_emul_mask(const struct wined3d_gl_info *gl_info)
+static unsigned int nvrc_fragment_get_emul_mask(const struct wined3d_adapter *adapter)
 {
     return GL_EXT_EMUL_ARB_MULTITEXTURE | GL_EXT_EMUL_EXT_FOG_COORD;
 }
