@@ -36,7 +36,9 @@
 WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 
 static const GUID MEDIASUBTYPE_CVID = {mmioFOURCC('c','v','i','d'), 0x0000, 0x0010, {0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}};
+static const GUID MEDIASUBTYPE_VC1S = {mmioFOURCC('V','C','1','S'), 0x0000, 0x0010, {0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}};
 static const GUID MEDIASUBTYPE_MP3  = {WAVE_FORMAT_MPEGLAYER3, 0x0000, 0x0010, {0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}};
+static const GUID MEDIASUBTYPE_WMV_Unknown = {0x7ce12ca9, 0xbfbf, 0x43d9, {0x9d, 0x00, 0x82, 0xb8, 0xed, 0x54, 0x31, 0x6b}};
 
 struct parser
 {
@@ -50,13 +52,25 @@ struct parser
     unsigned int source_count;
     BOOL enum_sink_first;
 
-    struct wg_parser *wg_parser;
+    wg_parser_t wg_parser;
+
+    /* This protects the "streaming" and "flushing" fields, accessed by both
+     * the application and streaming threads.
+     * We cannot use the filter lock for this, since that is held while waiting
+     * for the streaming thread, and hence the streaming thread cannot take the
+     * filter lock.
+     * This lock must not be acquired before acquiring the filter lock or
+     * flushing_cs. */
+    CRITICAL_SECTION streaming_cs;
+    CONDITION_VARIABLE flushing_cv;
 
     /* FIXME: It would be nice to avoid duplicating these with strmbase.
      * However, synchronization is tricky; we need access to be protected by a
      * separate lock. */
     bool streaming, sink_connected;
 
+    bool flushing;
+
     HANDLE read_thread;
 
     BOOL (*init_gst)(struct parser *filter);
@@ -69,7 +83,7 @@ struct parser_source
     struct strmbase_source pin;
     IQualityControl IQualityControl_iface;
 
-    struct wg_parser_stream *wg_stream;
+    wg_parser_stream_t wg_stream;
 
     SourceSeeking seek;
 
@@ -95,7 +109,7 @@ static const IMediaSeekingVtbl GST_Seeking_Vtbl;
 static const IQualityControlVtbl GSTOutPin_QualityControl_Vtbl;
 
 static struct parser_source *create_pin(struct parser *filter,
-        struct wg_parser_stream *stream, const WCHAR *name);
+        wg_parser_stream_t stream, const WCHAR *name);
 static HRESULT GST_RemoveOutputPins(struct parser *This);
 static HRESULT WINAPI GST_ChangeCurrent(IMediaSeeking *iface);
 static HRESULT WINAPI GST_ChangeStop(IMediaSeeking *iface);
@@ -266,46 +280,50 @@ static bool amt_from_wg_format_audio_mpeg1(AM_MEDIA_TYPE *mt, const struct wg_fo
 
 #define ALIGN(n, alignment) (((n) + (alignment) - 1) & ~((alignment) - 1))
 
+static unsigned int wg_format_get_max_size_video_raw(enum wg_video_format format, unsigned int width, unsigned int height)
+{
+    switch (format)
+    {
+        case WG_VIDEO_FORMAT_BGRA:
+        case WG_VIDEO_FORMAT_BGRx:
+        case WG_VIDEO_FORMAT_AYUV:
+            return width * height * 4;
+
+        case WG_VIDEO_FORMAT_BGR:
+            return ALIGN(width * 3, 4) * height;
+
+        case WG_VIDEO_FORMAT_RGB15:
+        case WG_VIDEO_FORMAT_RGB16:
+        case WG_VIDEO_FORMAT_UYVY:
+        case WG_VIDEO_FORMAT_YUY2:
+        case WG_VIDEO_FORMAT_YVYU:
+            return ALIGN(width * 2, 4) * height;
+
+        case WG_VIDEO_FORMAT_I420:
+        case WG_VIDEO_FORMAT_YV12:
+            return ALIGN(width, 4) * ALIGN(height, 2) /* Y plane */
+                    + 2 * ALIGN((width + 1) / 2, 4) * ((height + 1) / 2); /* U and V planes */
+
+        case WG_VIDEO_FORMAT_NV12:
+            return ALIGN(width, 4) * ALIGN(height, 2) /* Y plane */
+                    + ALIGN(width, 4) * ((height + 1) / 2); /* U/V plane */
+
+        case WG_VIDEO_FORMAT_UNKNOWN:
+            FIXME("Cannot guess maximum sample size for unknown video format.\n");
+            return 0;
+    }
+
+    assert(0);
+    return 0;
+}
+
 unsigned int wg_format_get_max_size(const struct wg_format *format)
 {
     switch (format->major_type)
     {
         case WG_MAJOR_TYPE_VIDEO:
-        {
-            unsigned int width = format->u.video.width, height = format->u.video.height;
-
-            switch (format->u.video.format)
-            {
-                case WG_VIDEO_FORMAT_BGRA:
-                case WG_VIDEO_FORMAT_BGRx:
-                case WG_VIDEO_FORMAT_AYUV:
-                    return width * height * 4;
-
-                case WG_VIDEO_FORMAT_BGR:
-                    return ALIGN(width * 3, 4) * height;
-
-                case WG_VIDEO_FORMAT_RGB15:
-                case WG_VIDEO_FORMAT_RGB16:
-                case WG_VIDEO_FORMAT_UYVY:
-                case WG_VIDEO_FORMAT_YUY2:
-                case WG_VIDEO_FORMAT_YVYU:
-                    return ALIGN(width * 2, 4) * height;
-
-                case WG_VIDEO_FORMAT_I420:
-                case WG_VIDEO_FORMAT_YV12:
-                    return ALIGN(width, 4) * ALIGN(height, 2) /* Y plane */
-                            + 2 * ALIGN((width + 1) / 2, 4) * ((height + 1) / 2); /* U and V planes */
-
-                case WG_VIDEO_FORMAT_NV12:
-                    return ALIGN(width, 4) * ALIGN(height, 2) /* Y plane */
-                            + ALIGN(width, 4) * ((height + 1) / 2); /* U/V plane */
-
-                case WG_VIDEO_FORMAT_UNKNOWN:
-                    FIXME("Cannot guess maximum sample size for unknown video format.\n");
-                    return 0;
-            }
-            break;
-        }
+            return wg_format_get_max_size_video_raw(format->u.video.format,
+                    format->u.video.width, abs(format->u.video.height));
 
         case WG_MAJOR_TYPE_VIDEO_CINEPAK:
             /* Both ffmpeg's encoder and a Cinepak file seen in the wild report
@@ -313,6 +331,13 @@ unsigned int wg_format_get_max_size(const struct wg_format *format)
              * but as long as every sample fits into our allocator, we're fine. */
             return format->u.video_cinepak.width * format->u.video_cinepak.height * 3;
 
+        case WG_MAJOR_TYPE_VIDEO_MPEG1:
+            /* Estimated max size of a compressed video frame.
+             * There's no way to no way to know the real upper bound,
+             * so let's just use the decompressed size and hope it works. */
+            return wg_format_get_max_size_video_raw(WG_VIDEO_FORMAT_YV12,
+                    format->u.video_mpeg1.width, format->u.video_mpeg1.height);
+
         case WG_MAJOR_TYPE_AUDIO:
         {
             unsigned int rate = format->u.audio.rate, channels = format->u.audio.channels;
@@ -362,6 +387,8 @@ unsigned int wg_format_get_max_size(const struct wg_format *format)
         case WG_MAJOR_TYPE_AUDIO_MPEG4:
         case WG_MAJOR_TYPE_AUDIO_WMA:
         case WG_MAJOR_TYPE_VIDEO_H264:
+        case WG_MAJOR_TYPE_VIDEO_WMV:
+        case WG_MAJOR_TYPE_VIDEO_INDEO:
             FIXME("Format %u not implemented!\n", format->major_type);
             return 0;
 
@@ -469,7 +496,7 @@ static bool amt_from_wg_format_video(AM_MEDIA_TYPE *mt, const struct wg_format *
 
     if (wm)
     {
-        SetRect(&video_format->rcSource, 0, 0, format->u.video.width, format->u.video.height);
+        SetRect(&video_format->rcSource, 0, 0, format->u.video.width, abs(format->u.video.height));
         video_format->rcTarget = video_format->rcSource;
     }
     if ((frame_time = MulDiv(10000000, format->u.video.fps_d, format->u.video.fps_n)) != -1)
@@ -477,6 +504,8 @@ static bool amt_from_wg_format_video(AM_MEDIA_TYPE *mt, const struct wg_format *
     video_format->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
     video_format->bmiHeader.biWidth = format->u.video.width;
     video_format->bmiHeader.biHeight = format->u.video.height;
+    if (wg_video_format_is_rgb(format->u.video.format))
+        video_format->bmiHeader.biHeight = -format->u.video.height;
     video_format->bmiHeader.biPlanes = 1;
     video_format->bmiHeader.biBitCount = wg_video_format_get_depth(format->u.video.format);
     video_format->bmiHeader.biCompression = wg_video_format_get_compression(format->u.video.format);
@@ -523,6 +552,90 @@ static bool amt_from_wg_format_video_cinepak(AM_MEDIA_TYPE *mt, const struct wg_
     return true;
 }
 
+static bool amt_from_wg_format_video_wmv(AM_MEDIA_TYPE *mt, const struct wg_format *format)
+{
+    VIDEOINFO *video_format;
+    uint32_t frame_time;
+    const GUID *subtype;
+
+    if (!(video_format = CoTaskMemAlloc(sizeof(*video_format))))
+        return false;
+
+    switch (format->u.video_wmv.format)
+    {
+        case WG_WMV_VIDEO_FORMAT_WMV1:
+            subtype = &MEDIASUBTYPE_WMV1;
+            break;
+        case WG_WMV_VIDEO_FORMAT_WMV2:
+            subtype = &MEDIASUBTYPE_WMV2;
+            break;
+        case WG_WMV_VIDEO_FORMAT_WMV3:
+            subtype = &MEDIASUBTYPE_WMV3;
+            break;
+        case WG_WMV_VIDEO_FORMAT_WMVA:
+            subtype = &MEDIASUBTYPE_WMVA;
+            break;
+        case WG_WMV_VIDEO_FORMAT_WVC1:
+            subtype = &MEDIASUBTYPE_WVC1;
+            break;
+        default:
+            WARN("Invalid WMV format %u.\n", format->u.video_wmv.format);
+            return false;
+    }
+
+    mt->majortype = MEDIATYPE_Video;
+    mt->subtype = *subtype;
+    mt->bFixedSizeSamples = FALSE;
+    mt->bTemporalCompression = TRUE;
+    mt->lSampleSize = 0;
+    mt->formattype = FORMAT_VideoInfo;
+    mt->cbFormat = sizeof(VIDEOINFOHEADER);
+    mt->pbFormat = (BYTE *)video_format;
+
+    memset(video_format, 0, sizeof(*video_format));
+    SetRect(&video_format->rcSource, 0, 0, format->u.video_wmv.width, format->u.video_wmv.height);
+    video_format->rcTarget = video_format->rcSource;
+    if ((frame_time = MulDiv(10000000, format->u.video_wmv.fps_d, format->u.video_wmv.fps_n)) != -1)
+        video_format->AvgTimePerFrame = frame_time;
+    video_format->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
+    video_format->bmiHeader.biWidth = format->u.video_wmv.width;
+    video_format->bmiHeader.biHeight = format->u.video_wmv.height;
+    video_format->bmiHeader.biPlanes = 1;
+    video_format->bmiHeader.biCompression = mt->subtype.Data1;
+
+    return true;
+}
+
+static bool amt_from_wg_format_video_mpeg1(AM_MEDIA_TYPE *mt, const struct wg_format *format)
+{
+    MPEG1VIDEOINFO *video_format;
+    uint32_t frame_time;
+
+    if (!(video_format = CoTaskMemAlloc(sizeof(*video_format))))
+        return false;
+
+    mt->majortype = MEDIATYPE_Video;
+    mt->subtype = MEDIASUBTYPE_MPEG1Payload;
+    mt->bTemporalCompression = TRUE;
+    mt->lSampleSize = 1;
+    mt->formattype = FORMAT_MPEGVideo;
+    mt->cbFormat = sizeof(MPEG1VIDEOINFO);
+    mt->pbFormat = (BYTE *)video_format;
+
+    memset(video_format, 0, sizeof(*video_format));
+    if ((frame_time = MulDiv(10000000, format->u.video_mpeg1.fps_d, format->u.video_mpeg1.fps_n)) != -1)
+        video_format->hdr.AvgTimePerFrame = frame_time;
+    video_format->hdr.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
+    video_format->hdr.bmiHeader.biWidth = format->u.video_mpeg1.width;
+    video_format->hdr.bmiHeader.biHeight = format->u.video_mpeg1.height;
+    video_format->hdr.bmiHeader.biPlanes = 1;
+    video_format->hdr.bmiHeader.biBitCount = 12;
+    video_format->hdr.bmiHeader.biCompression = mt->subtype.Data1;
+    video_format->hdr.bmiHeader.biSizeImage = wg_format_get_max_size(format);
+
+    return true;
+}
+
 bool amt_from_wg_format(AM_MEDIA_TYPE *mt, const struct wg_format *format, bool wm)
 {
     memset(mt, 0, sizeof(*mt));
@@ -532,6 +645,7 @@ bool amt_from_wg_format(AM_MEDIA_TYPE *mt, const struct wg_format *format, bool
     case WG_MAJOR_TYPE_AUDIO_MPEG4:
     case WG_MAJOR_TYPE_AUDIO_WMA:
     case WG_MAJOR_TYPE_VIDEO_H264:
+    case WG_MAJOR_TYPE_VIDEO_INDEO:
         FIXME("Format %u not implemented!\n", format->major_type);
         /* fallthrough */
     case WG_MAJOR_TYPE_UNKNOWN:
@@ -548,6 +662,12 @@ bool amt_from_wg_format(AM_MEDIA_TYPE *mt, const struct wg_format *format, bool
 
     case WG_MAJOR_TYPE_VIDEO_CINEPAK:
         return amt_from_wg_format_video_cinepak(mt, format);
+
+    case WG_MAJOR_TYPE_VIDEO_WMV:
+        return amt_from_wg_format_video_wmv(mt, format);
+
+    case WG_MAJOR_TYPE_VIDEO_MPEG1:
+        return amt_from_wg_format_video_mpeg1(mt, format);
     }
 
     assert(0);
@@ -715,6 +835,8 @@ static bool amt_to_wg_format_video(const AM_MEDIA_TYPE *mt, struct wg_format *fo
         if (IsEqualGUID(&mt->subtype, format_map[i].subtype))
         {
             format->u.video.format = format_map[i].format;
+            if (wg_video_format_is_rgb(format->u.video.format))
+                format->u.video.height = -format->u.video.height;
             return true;
         }
     }
@@ -723,12 +845,87 @@ static bool amt_to_wg_format_video(const AM_MEDIA_TYPE *mt, struct wg_format *fo
     return false;
 }
 
+static bool amt_to_wg_format_video_wmv(const AM_MEDIA_TYPE *mt, struct wg_format *format)
+{
+    const VIDEOINFOHEADER *video_format = (const VIDEOINFOHEADER *)mt->pbFormat;
+
+    if (!IsEqualGUID(&mt->formattype, &FORMAT_VideoInfo))
+    {
+        FIXME("Unknown format type %s.\n", debugstr_guid(&mt->formattype));
+        return false;
+    }
+    if (mt->cbFormat < sizeof(VIDEOINFOHEADER) || !mt->pbFormat)
+    {
+        ERR("Unexpected format size %lu.\n", mt->cbFormat);
+        return false;
+    }
+
+    format->major_type = WG_MAJOR_TYPE_VIDEO_WMV;
+    format->u.video_wmv.width = video_format->bmiHeader.biWidth;
+    format->u.video_wmv.height = video_format->bmiHeader.biHeight;
+    format->u.video_wmv.fps_n = 10000000;
+    format->u.video_wmv.fps_d = video_format->AvgTimePerFrame;
+
+    if (IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WMV1))
+        format->u.video_wmv.format = WG_WMV_VIDEO_FORMAT_WMV1;
+    else if (IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WMV2))
+        format->u.video_wmv.format = WG_WMV_VIDEO_FORMAT_WMV2;
+    else if (IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WMV3))
+        format->u.video_wmv.format = WG_WMV_VIDEO_FORMAT_WMV3;
+    else if (IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WMVA))
+        format->u.video_wmv.format = WG_WMV_VIDEO_FORMAT_WMVA;
+    else if (IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WVC1))
+        format->u.video_wmv.format = WG_WMV_VIDEO_FORMAT_WVC1;
+    else
+        format->u.video_wmv.format = WG_WMV_VIDEO_FORMAT_UNKNOWN;
+
+    return true;
+}
+
+static bool amt_to_wg_format_video_mpeg1(const AM_MEDIA_TYPE *mt, struct wg_format *format)
+{
+    const MPEG1VIDEOINFO *video_format = (const MPEG1VIDEOINFO *)mt->pbFormat;
+
+    if (!IsEqualGUID(&mt->formattype, &FORMAT_MPEGVideo))
+    {
+        FIXME("Unknown format type %s.\n", debugstr_guid(&mt->formattype));
+        return false;
+    }
+    if (mt->cbFormat < sizeof(VIDEOINFOHEADER) || !mt->pbFormat)
+    {
+        ERR("Unexpected format size %lu.\n", mt->cbFormat);
+        return false;
+    }
+
+    format->major_type = WG_MAJOR_TYPE_VIDEO_MPEG1;
+    format->u.video_mpeg1.width = video_format->hdr.bmiHeader.biWidth;
+    format->u.video_mpeg1.height = video_format->hdr.bmiHeader.biHeight;
+    format->u.video_mpeg1.fps_n = 10000000;
+    format->u.video_mpeg1.fps_d = video_format->hdr.AvgTimePerFrame;
+
+    return true;
+}
+
 bool amt_to_wg_format(const AM_MEDIA_TYPE *mt, struct wg_format *format)
 {
     memset(format, 0, sizeof(*format));
 
     if (IsEqualGUID(&mt->majortype, &MEDIATYPE_Video))
+    {
+        if (IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WMV1)
+                || IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WMV2)
+                || IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WMVA)
+                || IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WMVP)
+                || IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WVP2)
+                || IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WMV_Unknown)
+                || IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WVC1)
+                || IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_WMV3)
+                || IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_VC1S))
+            return amt_to_wg_format_video_wmv(mt, format);
+        if (IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_MPEG1Payload))
+            return amt_to_wg_format_video_mpeg1(mt, format);
         return amt_to_wg_format_video(mt, format);
+    }
     if (IsEqualGUID(&mt->majortype, &MEDIATYPE_Audio))
     {
         if (IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_MPEG1AudioPayload))
@@ -925,10 +1122,23 @@ static DWORD CALLBACK stream_thread(void *arg)
 
     TRACE("Starting streaming thread for pin %p.\n", pin);
 
-    while (filter->streaming)
+    for (;;)
     {
         struct wg_parser_buffer buffer;
 
+        EnterCriticalSection(&filter->streaming_cs);
+
+        while (filter->flushing)
+            SleepConditionVariableCS(&filter->flushing_cv, &filter->streaming_cs, INFINITE);
+
+        if (!filter->streaming)
+        {
+            LeaveCriticalSection(&filter->streaming_cs);
+            break;
+        }
+
+        LeaveCriticalSection(&filter->streaming_cs);
+
         EnterCriticalSection(&pin->flushing_cs);
 
         if (pin->eos)
@@ -1048,6 +1258,9 @@ static void parser_destroy(struct strmbase_filter *iface)
 
     wg_parser_destroy(filter->wg_parser);
 
+    filter->streaming_cs.DebugInfo->Spare[0] = 0;
+    DeleteCriticalSection(&filter->streaming_cs);
+
     strmbase_sink_cleanup(&filter->sink);
     strmbase_filter_cleanup(&filter->filter);
     free(filter);
@@ -1120,7 +1333,9 @@ static HRESULT parser_cleanup_stream(struct strmbase_filter *iface)
     if (!filter->sink_connected)
         return S_OK;
 
+    EnterCriticalSection(&filter->streaming_cs);
     filter->streaming = false;
+    LeaveCriticalSection(&filter->streaming_cs);
 
     for (i = 0; i < filter->source_count; ++i)
     {
@@ -1220,7 +1435,7 @@ static const struct strmbase_sink_ops sink_ops =
 
 static BOOL decodebin_parser_filter_init_gst(struct parser *filter)
 {
-    struct wg_parser *parser = filter->wg_parser;
+    wg_parser_t parser = filter->wg_parser;
     unsigned int i, stream_count;
     WCHAR source_name[20];
 
@@ -1281,6 +1496,9 @@ static HRESULT decodebin_parser_source_get_media_type(struct parser_source *pin,
     if (format.major_type == WG_MAJOR_TYPE_VIDEO && index < ARRAY_SIZE(video_formats))
     {
         format.u.video.format = video_formats[index];
+        /* Downstream filters probably expect RGB video to be bottom-up. */
+        if (format.u.video.height > 0 && wg_video_format_is_rgb(video_formats[index]))
+            format.u.video.height = -format.u.video.height;
         if (!amt_from_wg_format(mt, &format, false))
             return E_OUTOFMEMORY;
         return S_OK;
@@ -1296,28 +1514,35 @@ static HRESULT decodebin_parser_source_get_media_type(struct parser_source *pin,
     return VFW_S_NO_MORE_ITEMS;
 }
 
-static BOOL parser_init_gstreamer(void)
+static HRESULT parser_create(enum wg_parser_type type, BOOL output_compressed, struct parser **parser)
 {
-    if (!init_gstreamer())
-        return FALSE;
-    return TRUE;
+    struct parser *object;
+
+    if (!(object = calloc(1, sizeof(*object))))
+        return E_OUTOFMEMORY;
+
+    if (!(object->wg_parser = wg_parser_create(type, output_compressed)))
+    {
+        free(object);
+        return E_OUTOFMEMORY;
+    }
+
+    InitializeCriticalSection(&object->streaming_cs);
+    object->streaming_cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": parser.streaming_cs");
+
+    InitializeConditionVariable(&object->flushing_cv);
+
+    *parser = object;
+    return S_OK;
 }
 
 HRESULT decodebin_parser_create(IUnknown *outer, IUnknown **out)
 {
     struct parser *object;
+    HRESULT hr;
 
-    if (!parser_init_gstreamer())
-        return E_FAIL;
-
-    if (!(object = calloc(1, sizeof(*object))))
-        return E_OUTOFMEMORY;
-
-    if (!(object->wg_parser = wg_parser_create(WG_PARSER_DECODEBIN, false)))
-    {
-        free(object);
-        return E_OUTOFMEMORY;
-    }
+    if (FAILED(hr = parser_create(WG_PARSER_DECODEBIN, FALSE, &object)))
+        return hr;
 
     strmbase_filter_init(&object->filter, outer, &CLSID_decodebin_parser, &filter_ops);
     strmbase_sink_init(&object->sink, &object->filter, L"input pin", &sink_ops, NULL);
@@ -1356,17 +1581,56 @@ static ULONG WINAPI stream_select_Release(IAMStreamSelect *iface)
 
 static HRESULT WINAPI stream_select_Count(IAMStreamSelect *iface, DWORD *count)
 {
-    FIXME("iface %p, count %p, stub!\n", iface, count);
-    return E_NOTIMPL;
+    struct parser *filter = impl_from_IAMStreamSelect(iface);
+    TRACE("filter %p, count %p\n", filter, count);
+    EnterCriticalSection(&filter->filter.filter_cs);
+    if (filter->sink.pin.peer)
+        *count = wg_parser_get_stream_count(filter->wg_parser);
+    else
+        *count = 0;
+    LeaveCriticalSection(&filter->filter.filter_cs);
+    return S_OK;
 }
 
 static HRESULT WINAPI stream_select_Info(IAMStreamSelect *iface, LONG index,
         AM_MEDIA_TYPE **mt, DWORD *flags, LCID *lcid, DWORD *group, WCHAR **name,
         IUnknown **object, IUnknown **unknown)
 {
-    FIXME("iface %p, index %ld, mt %p, flags %p, lcid %p, group %p, name %p, object %p, unknown %p, stub!\n",
-            iface, index, mt, flags, lcid, group, name, object, unknown);
-    return E_NOTIMPL;
+    struct parser *filter = impl_from_IAMStreamSelect(iface);
+    HRESULT hr = S_OK;
+
+    FIXME("filter %p, index %ld, mt %p, flags %p, lcid %p, group %p, name %p, object %p, unknown %p, semi-stub!\n",
+            filter, index, mt, flags, lcid, group, name, object, unknown);
+    EnterCriticalSection(&filter->filter.filter_cs);
+
+    if (!filter->sink.pin.peer)
+    {
+        LeaveCriticalSection(&filter->filter.filter_cs);
+        return VFW_E_NOT_CONNECTED;
+    }
+    if (index < 0 || index >= filter->source_count)
+    {
+        LeaveCriticalSection(&filter->filter.filter_cs);
+        return S_FALSE;
+    }
+
+    if (mt)
+        *mt = CreateMediaType(&filter->sources[index]->pin.pin.mt);
+    if (flags) /* todo */
+        *flags = 0;
+    if (lcid) /* todo */
+        *lcid = 0;
+    if (group) /* todo */
+        *group = 0;
+    if (name) /* todo */
+        *name = NULL;
+    if (object) /* todo */
+        *object = NULL;
+    if (unknown)
+        *unknown = NULL;
+
+    LeaveCriticalSection(&filter->filter.filter_cs);
+    return hr;
 }
 
 static HRESULT WINAPI stream_select_Enable(IAMStreamSelect *iface, LONG index, DWORD flags)
@@ -1455,8 +1719,13 @@ static HRESULT WINAPI GST_Seeking_SetPositions(IMediaSeeking *iface,
             IAsyncReader_BeginFlush(filter->reader);
     }
 
-    /* Acquire the flushing locks. This blocks the streaming threads, and
-     * ensures the seek is serialized between flushes. */
+    /* Signal the streaming threads to "pause". */
+    EnterCriticalSection(&filter->streaming_cs);
+    filter->flushing = true;
+    LeaveCriticalSection(&filter->streaming_cs);
+
+    /* Acquire the flushing locks, to make sure the streaming threads really
+     * are paused. This ensures the seek is serialized between flushes. */
     for (i = 0; i < filter->source_count; ++i)
     {
         struct parser_source *flush_pin = filter->sources[i];
@@ -1499,6 +1768,12 @@ static HRESULT WINAPI GST_Seeking_SetPositions(IMediaSeeking *iface,
         }
     }
 
+    /* Signal the streaming threads to resume. */
+    EnterCriticalSection(&filter->streaming_cs);
+    filter->flushing = false;
+    LeaveCriticalSection(&filter->streaming_cs);
+    WakeConditionVariable(&filter->flushing_cv);
+
     return S_OK;
 }
 
@@ -1658,6 +1933,12 @@ static HRESULT WINAPI GSTOutPin_DecideBufferSize(struct strmbase_source *iface,
         VIDEOINFOHEADER *format = (VIDEOINFOHEADER *)pin->pin.pin.mt.pbFormat;
         buffer_size = format->bmiHeader.biSizeImage;
     }
+    else if (IsEqualGUID(&pin->pin.pin.mt.formattype, &FORMAT_MPEGVideo))
+    {
+        MPEG1VIDEOINFO *format = (MPEG1VIDEOINFO *)pin->pin.pin.mt.pbFormat;
+        buffer_size = format->hdr.bmiHeader.biSizeImage;
+        buffer_count = 8;
+    }
     else if (IsEqualGUID(&pin->pin.pin.mt.formattype, &FORMAT_WaveFormatEx)
             && (IsEqualGUID(&pin->pin.pin.mt.subtype, &MEDIASUBTYPE_PCM)
             || IsEqualGUID(&pin->pin.pin.mt.subtype, &MEDIASUBTYPE_IEEE_FLOAT)))
@@ -1710,7 +1991,7 @@ static const struct strmbase_source_ops source_ops =
 };
 
 static struct parser_source *create_pin(struct parser *filter,
-        struct wg_parser_stream *stream, const WCHAR *name)
+        wg_parser_stream_t stream, const WCHAR *name)
 {
     struct parser_source *pin, **new_array;
 
@@ -1796,9 +2077,7 @@ static const struct strmbase_sink_ops wave_parser_sink_ops =
 
 static BOOL wave_parser_filter_init_gst(struct parser *filter)
 {
-    struct wg_parser *parser = filter->wg_parser;
-
-    if (!create_pin(filter, wg_parser_get_stream(parser, 0), L"output"))
+    if (!create_pin(filter, wg_parser_get_stream(filter->wg_parser, 0), L"output"))
         return FALSE;
 
     return TRUE;
@@ -1834,18 +2113,10 @@ static HRESULT wave_parser_source_get_media_type(struct parser_source *pin,
 HRESULT wave_parser_create(IUnknown *outer, IUnknown **out)
 {
     struct parser *object;
+    HRESULT hr;
 
-    if (!parser_init_gstreamer())
-        return E_FAIL;
-
-    if (!(object = calloc(1, sizeof(*object))))
-        return E_OUTOFMEMORY;
-
-    if (!(object->wg_parser = wg_parser_create(WG_PARSER_WAVPARSE, false)))
-    {
-        free(object);
-        return E_OUTOFMEMORY;
-    }
+    if (FAILED(hr = parser_create(WG_PARSER_WAVPARSE, FALSE, &object)))
+        return hr;
 
     strmbase_filter_init(&object->filter, outer, &CLSID_WAVEParser, &filter_ops);
     strmbase_sink_init(&object->sink, &object->filter, L"input pin", &wave_parser_sink_ops, NULL);
@@ -1875,7 +2146,7 @@ static const struct strmbase_sink_ops avi_splitter_sink_ops =
 
 static BOOL avi_splitter_filter_init_gst(struct parser *filter)
 {
-    struct wg_parser *parser = filter->wg_parser;
+    wg_parser_t parser = filter->wg_parser;
     uint32_t i, stream_count;
     WCHAR source_name[20];
 
@@ -1920,18 +2191,10 @@ static HRESULT avi_splitter_source_get_media_type(struct parser_source *pin,
 HRESULT avi_splitter_create(IUnknown *outer, IUnknown **out)
 {
     struct parser *object;
+    HRESULT hr;
 
-    if (!parser_init_gstreamer())
-        return E_FAIL;
-
-    if (!(object = calloc(1, sizeof(*object))))
-        return E_OUTOFMEMORY;
-
-    if (!(object->wg_parser = wg_parser_create(WG_PARSER_AVIDEMUX, false)))
-    {
-        free(object);
-        return E_OUTOFMEMORY;
-    }
+    if (FAILED(hr = parser_create(WG_PARSER_AVIDEMUX, FALSE, &object)))
+        return hr;
 
     strmbase_filter_init(&object->filter, outer, &CLSID_AviSplitter, &filter_ops);
     strmbase_sink_init(&object->sink, &object->filter, L"input pin", &avi_splitter_sink_ops, NULL);
@@ -1948,10 +2211,10 @@ static HRESULT mpeg_splitter_sink_query_accept(struct strmbase_pin *iface, const
 {
     if (!IsEqualGUID(&mt->majortype, &MEDIATYPE_Stream))
         return S_FALSE;
-    if (IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_MPEG1Audio))
+    if (IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_MPEG1Audio)
+            || IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_MPEG1System))
         return S_OK;
     if (IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_MPEG1Video)
-            || IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_MPEG1System)
             || IsEqualGUID(&mt->subtype, &MEDIASUBTYPE_MPEG1VideoCD))
         FIXME("Unsupported subtype %s.\n", wine_dbgstr_guid(&mt->subtype));
     return S_FALSE;
@@ -1966,10 +2229,28 @@ static const struct strmbase_sink_ops mpeg_splitter_sink_ops =
 
 static BOOL mpeg_splitter_filter_init_gst(struct parser *filter)
 {
-    struct wg_parser *parser = filter->wg_parser;
+    wg_parser_t parser = filter->wg_parser;
+    unsigned int i, stream_count;
+    wg_parser_stream_t stream;
+    struct wg_format fmt;
 
-    if (!create_pin(filter, wg_parser_get_stream(parser, 0), L"Audio"))
-        return FALSE;
+    stream_count = wg_parser_get_stream_count(parser);
+    for (i = 0; i < stream_count; ++i)
+    {
+        stream = wg_parser_get_stream(parser, i);
+        wg_parser_stream_get_preferred_format(stream, &fmt);
+        if (fmt.major_type == WG_MAJOR_TYPE_VIDEO_MPEG1)
+        {
+            if (!create_pin(filter, wg_parser_get_stream(parser, i), L"Video"))
+                return FALSE;
+        }
+        else if (fmt.major_type == WG_MAJOR_TYPE_AUDIO_MPEG1)
+        {
+            if (!create_pin(filter, wg_parser_get_stream(parser, i), L"Audio"))
+                return FALSE;
+        }
+        else FIXME("unexpected format %u\n", fmt.major_type);
+    }
 
     return TRUE;
 }
@@ -2027,18 +2308,10 @@ static const struct strmbase_filter_ops mpeg_splitter_ops =
 HRESULT mpeg_splitter_create(IUnknown *outer, IUnknown **out)
 {
     struct parser *object;
+    HRESULT hr;
 
-    if (!parser_init_gstreamer())
-        return E_FAIL;
-
-    if (!(object = calloc(1, sizeof(*object))))
-        return E_OUTOFMEMORY;
-
-    if (!(object->wg_parser = wg_parser_create(WG_PARSER_MPEGAUDIOPARSE, false)))
-    {
-        free(object);
-        return E_OUTOFMEMORY;
-    }
+    if (FAILED(hr = parser_create(WG_PARSER_DECODEBIN, TRUE, &object)))
+        return hr;
 
     strmbase_filter_init(&object->filter, outer, &CLSID_MPEG1Splitter, &mpeg_splitter_ops);
     strmbase_sink_init(&object->sink, &object->filter, L"Input", &mpeg_splitter_sink_ops, NULL);
