@@ -26,17 +26,13 @@ enum macdrv_funcs
     unix_dnd_have_format,
     unix_dnd_release,
     unix_dnd_retain,
-    unix_ime_clear,
-    unix_ime_process_text_input,
-    unix_ime_using_input_method,
     unix_init,
     unix_notify_icon,
     unix_quit_result,
     unix_funcs_count
 };
 
-extern unixlib_handle_t macdrv_handle DECLSPEC_HIDDEN;
-#define MACDRV_CALL(func, params) __wine_unix_call(macdrv_handle, unix_ ## func, params)
+#define MACDRV_CALL(func, params) WINE_UNIX_CALL(unix_ ## func, params)
 
 /* macdrv_dnd_get_data params */
 struct dnd_get_data_params
@@ -61,17 +57,6 @@ struct dnd_have_format_params
     UINT format;
 };
 
-/* macdrv_ime_process_text_input params */
-struct process_text_input_params
-{
-    UINT vkey;
-    UINT scan;
-    UINT repeat;
-    const BYTE *key_state;
-    void *himc;
-    int *done;
-};
-
 /* macdrv_init params */
 struct localized_string
 {
@@ -88,7 +73,7 @@ struct init_params
 /* macdrv_notify_icon params */
 struct notify_icon_params
 {
-    DWORD msg;
+    unsigned int msg;
     struct _NOTIFYICONDATAW *data;
 };
 
@@ -106,8 +91,6 @@ enum macdrv_client_funcs
     client_func_dnd_query_drag,
     client_func_dnd_query_drop,
     client_func_dnd_query_exited,
-    client_func_ime_query_char_rect,
-    client_func_ime_set_text,
     client_func_last
 };
 
@@ -121,18 +104,6 @@ struct app_icon_entry
     UINT64 png;
 };
 
-struct app_icon_result
-{
-    UINT32 count;
-    struct app_icon_entry entries[64];
-};
-
-/* macdrv_app_icon params */
-struct app_icon_params
-{
-    UINT64 result; /* FIXME: Use NtCallbackReturn instead */
-};
-
 /* macdrv_app_quit_request params */
 struct app_quit_request_params
 {
@@ -165,34 +136,6 @@ struct dnd_query_exited_params
     UINT32 hwnd;
 };
 
-/* macdrv_ime_query_char_rect result */
-struct ime_query_char_rect_result
-{
-    RECT rect;
-    UINT32 location;
-    UINT32 length;
-};
-
-/* macdrv_ime_query_char_rect params */
-struct ime_query_char_rect_params
-{
-    UINT32 hwnd;
-    UINT32 location;
-    UINT64 data;
-    UINT64 result; /* FIXME: Use NtCallbackReturn instead */
-    UINT32 length;
-};
-
-/* macdrv_ime_set_text params */
-struct ime_set_text_params
-{
-    UINT32 hwnd;
-    UINT32 cursor_pos;
-    UINT64 data;
-    UINT32 complete;
-    WCHAR text[1];
-};
-
 static inline void *param_ptr(UINT64 param)
 {
     return (void *)(UINT_PTR)param;
