@@ -125,9 +125,12 @@ struct wine_phys_dev
     VkPhysicalDevice handle; /* client physical device */
     VkPhysicalDevice phys_dev; /* native physical device */
 
+    VkPhysicalDeviceMemoryProperties memory_properties;
     VkExtensionProperties *extensions;
     uint32_t extension_count;
 
+    uint32_t external_memory_align;
+
     struct wine_vk_mapping mapping;
 };
 
@@ -169,6 +172,17 @@ static inline struct wine_cmd_pool *wine_cmd_pool_from_handle(VkCommandPool hand
     return (struct wine_cmd_pool *)(uintptr_t)client_ptr->unix_handle;
 }
 
+struct wine_device_memory
+{
+    VkDeviceMemory memory;
+    void *mapping;
+};
+
+static inline struct wine_device_memory *wine_device_memory_from_handle(VkDeviceMemory handle)
+{
+    return (struct wine_device_memory *)(uintptr_t)handle;
+}
+
 struct wine_debug_utils_messenger
 {
     struct wine_instance *instance; /* parent */
@@ -223,20 +237,17 @@ static inline VkSurfaceKHR wine_surface_to_handle(struct wine_surface *surface)
     return (VkSurfaceKHR)(uintptr_t)surface;
 }
 
-BOOL wine_vk_device_extension_supported(const char *name) DECLSPEC_HIDDEN;
-BOOL wine_vk_instance_extension_supported(const char *name) DECLSPEC_HIDDEN;
+BOOL wine_vk_device_extension_supported(const char *name);
+BOOL wine_vk_instance_extension_supported(const char *name);
 
-BOOL wine_vk_is_type_wrapped(VkObjectType type) DECLSPEC_HIDDEN;
+BOOL wine_vk_is_type_wrapped(VkObjectType type);
 
-NTSTATUS init_vulkan(void *args) DECLSPEC_HIDDEN;
-NTSTATUS init_vulkan32(void *args) DECLSPEC_HIDDEN;
+NTSTATUS init_vulkan(void *args);
 
-NTSTATUS WINAPI vk_direct_unix_call(unixlib_handle_t handle, unsigned int code, void *arg) DECLSPEC_HIDDEN;
-
-NTSTATUS vk_is_available_instance_function(void *arg) DECLSPEC_HIDDEN;
-NTSTATUS vk_is_available_device_function(void *arg) DECLSPEC_HIDDEN;
-NTSTATUS vk_is_available_instance_function32(void *arg) DECLSPEC_HIDDEN;
-NTSTATUS vk_is_available_device_function32(void *arg) DECLSPEC_HIDDEN;
+NTSTATUS vk_is_available_instance_function(void *arg);
+NTSTATUS vk_is_available_device_function(void *arg);
+NTSTATUS vk_is_available_instance_function32(void *arg);
+NTSTATUS vk_is_available_device_function32(void *arg);
 
 struct conversion_context
 {
@@ -276,6 +287,27 @@ static inline void *conversion_context_alloc(struct conversion_context *pool, si
     }
 }
 
+struct wine_deferred_operation
+{
+    VkDeferredOperationKHR deferred_operation; /* native handle */
+
+    struct conversion_context ctx; /* to keep params alive. */
+
+    struct wine_vk_mapping mapping;
+};
+
+static inline struct wine_deferred_operation *wine_deferred_operation_from_handle(
+        VkDeferredOperationKHR handle)
+{
+    return (struct wine_deferred_operation *)(uintptr_t)handle;
+}
+
+static inline VkDeferredOperationKHR wine_deferred_operation_to_handle(
+        struct wine_deferred_operation *deferred_operation)
+{
+    return (VkDeferredOperationKHR)(uintptr_t)deferred_operation;
+}
+
 typedef UINT32 PTR32;
 
 typedef struct
