@@ -32,9 +32,8 @@
 #include <stdarg.h>
 #include <unistd.h>
 #include <sys/types.h>
-#ifdef HAVE_SYS_SOCKET_H
-# include <sys/socket.h>
-#endif
+#include <sys/socket.h>
+#include <sys/time.h>
 #ifdef HAVE_NETDB_H
 # include <netdb.h>
 #endif
@@ -64,12 +63,10 @@
 # include <ifaddrs.h>
 #endif
 #include <poll.h>
-#ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
-#endif
 
 #ifdef HAVE_NETIPX_IPX_H
 # include <netipx/ipx.h>
+# define HAS_IPX
 #elif defined(HAVE_LINUX_IPX_H)
 # ifdef HAVE_ASM_TYPES_H
 #  include <asm/types.h>
@@ -78,9 +75,9 @@
 #  include <linux/types.h>
 # endif
 # include <linux/ipx.h>
-#endif
-#if defined(SOL_IPX) || defined(SO_DEFAULT_HEADERS)
-# define HAS_IPX
+# ifdef SOL_IPX
+#  define HAS_IPX
+# endif
 #endif
 
 #ifdef HAVE_LINUX_IRDA_H
@@ -106,6 +103,7 @@
 #include "ws2_32_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(winsock);
+WINE_DECLARE_DEBUG_CHANNEL(winediag);
 
 #ifndef HAVE_LINUX_GETHOSTBYNAME_R_6
 static pthread_mutex_t host_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -171,6 +169,51 @@ static const int ip_protocol_map[][2] =
 
 #undef MAP
 
+static pthread_once_t hash_init_once = PTHREAD_ONCE_INIT;
+static BYTE byte_hash[256];
+
+static void init_hash(void)
+{
+    unsigned i, index;
+    NTSTATUS status;
+    BYTE *buf, tmp;
+    ULONG buf_len;
+
+    for (i = 0; i < sizeof(byte_hash); ++i)
+        byte_hash[i] = i;
+
+    buf_len = sizeof(SYSTEM_INTERRUPT_INFORMATION) * NtCurrentTeb()->Peb->NumberOfProcessors;
+    if (!(buf = malloc( buf_len )))
+    {
+        ERR( "No memory.\n" );
+        return;
+    }
+
+    for (i = 0; i < sizeof(byte_hash) - 1; ++i)
+    {
+        if (!(i % buf_len) && (status = NtQuerySystemInformation( SystemInterruptInformation, buf,
+                                                                  buf_len, &buf_len )))
+        {
+            ERR( "Failed to get random bytes.\n" );
+            free( buf );
+            return;
+        }
+        index = i + buf[i % buf_len] % (sizeof(byte_hash) - i);
+        tmp = byte_hash[index];
+        byte_hash[index] = byte_hash[i];
+        byte_hash[i] = tmp;
+    }
+    free( buf );
+}
+
+static void hash_random( BYTE *d, const BYTE *s, unsigned int len )
+{
+    unsigned int i;
+
+    for (i = 0; i < len; ++i)
+        d[i] = byte_hash[s[i]];
+}
+
 static int addrinfo_flags_from_unix( int flags )
 {
     int ws_flags = 0;
@@ -422,6 +465,7 @@ static int addrinfo_err_from_unix( int err )
         case EAI_SERVICE:   return WS_EAI_SERVICE;
         case EAI_SOCKTYPE:  return WS_EAI_SOCKTYPE;
         case EAI_SYSTEM:
+            if (errno == EBUSY) ERR_(winediag)("getaddrinfo() returned EBUSY. You may be missing a libnss plugin\n");
             /* some broken versions of glibc return EAI_SYSTEM and set errno to
              * 0 instead of returning EAI_NONAME */
             return errno ? errno_from_unix( errno ) : WS_EAI_NONAME;
@@ -756,6 +800,7 @@ static NTSTATUS unix_getaddrinfo( void *args )
         dst->ai_addrlen = sockaddr_from_unix( (const union unix_sockaddr *)src->ai_addr, NULL, 0 );
         dst->ai_addr = next;
         sockaddr_from_unix( (const union unix_sockaddr *)src->ai_addr, dst->ai_addr, dst->ai_addrlen );
+        dst->ai_next = NULL;
         next = (char *)dst->ai_addr + dst->ai_addrlen;
 
         if (dst == params->info || !addrinfo_in_list( params->info, dst ))
@@ -767,8 +812,6 @@ static NTSTATUS unix_getaddrinfo( void *args )
         }
     }
 
-    dst->ai_next = NULL;
-
     freeaddrinfo( unix_info );
     return 0;
 #else
@@ -783,7 +826,7 @@ static int hostent_from_unix( const struct hostent *unix_host, struct WS_hostent
     unsigned int needed_size = sizeof( struct WS_hostent ), alias_count = 0, addr_count = 0, i;
     char *p;
 
-    needed_size += strlen( unix_host->h_name );
+    needed_size += strlen( unix_host->h_name ) + 1;
 
     for (alias_count = 0; unix_host->h_aliases[alias_count] != NULL; ++alias_count)
         needed_size += sizeof(char *) + strlen( unix_host->h_aliases[alias_count] ) + 1;
@@ -870,9 +913,9 @@ static NTSTATUS unix_gethostbyaddr( void *args )
         }
 
         if (!unix_host)
-            return (locerr < 0 ? errno_from_unix( errno ) : host_errno_from_unix( locerr ));
-
-        ret = hostent_from_unix( unix_host, params->host, params->size );
+            ret = (locerr < 0 ? errno_from_unix( errno ) : host_errno_from_unix( locerr ));
+        else
+            ret = hostent_from_unix( unix_host, params->host, params->size );
 
         free( unix_buffer );
         return ret;
@@ -894,6 +937,44 @@ static NTSTATUS unix_gethostbyaddr( void *args )
 #endif
 }
 
+static int compare_addrs_hashed( const void *a1, const void *a2, int addr_len )
+{
+    char a1_hashed[16], a2_hashed[16];
+
+    assert( addr_len <= sizeof(a1_hashed) );
+    hash_random( (BYTE *)a1_hashed, a1, addr_len );
+    hash_random( (BYTE *)a2_hashed, a2, addr_len );
+    return memcmp( a1_hashed, a2_hashed, addr_len );
+}
+
+static void sort_addrs_hashed( struct hostent *host )
+{
+    /* On Unix gethostbyname() may return IP addresses in random order on each call. On Windows the order of
+     * IP addresses is not determined as well but it is the same on consequent calls (changes after network
+     * resets and probably DNS timeout expiration).
+     * Life is Strange Remastered depends on gethostbyname() returning IP addresses in the same order to reuse
+     * the established TLS connection and avoid timeouts that happen in game when establishing multiple extra TLS
+     * connections.
+     * Just sorting the addresses would break server load balancing provided by gethostbyname(), so randomize the
+     * sort once per process. */
+    unsigned int i, j;
+    char *tmp;
+
+    pthread_once( &hash_init_once, init_hash );
+
+    for (i = 0; host->h_addr_list[i]; ++i)
+    {
+        for (j = i + 1; host->h_addr_list[j]; ++j)
+        {
+            if (compare_addrs_hashed( host->h_addr_list[j], host->h_addr_list[i], host->h_length ) < 0)
+            {
+                tmp = host->h_addr_list[j];
+                host->h_addr_list[j] = host->h_addr_list[i];
+                host->h_addr_list[i] = tmp;
+            }
+        }
+    }
+}
 
 #ifdef HAVE_LINUX_GETHOSTBYNAME_R_6
 static NTSTATUS unix_gethostbyname( void *args )
@@ -920,9 +1001,14 @@ static NTSTATUS unix_gethostbyname( void *args )
     }
 
     if (!unix_host)
-        return (locerr < 0 ? errno_from_unix( errno ) : host_errno_from_unix( locerr ));
-
-    ret = hostent_from_unix( unix_host, params->host, params->size );
+    {
+        ret = (locerr < 0 ? errno_from_unix( errno ) : host_errno_from_unix( locerr ));
+    }
+    else
+    {
+        sort_addrs_hashed( unix_host );
+        ret = hostent_from_unix( unix_host, params->host, params->size );
+    }
 
     free( unix_buffer );
     return ret;
@@ -943,6 +1029,7 @@ static NTSTATUS unix_gethostbyname( void *args )
         return ret;
     }
 
+    sort_addrs_hashed( unix_host );
     ret = hostent_from_unix( unix_host, params->host, params->size );
 
     pthread_mutex_unlock( &host_mutex );
@@ -983,3 +1070,287 @@ const unixlib_entry_t __wine_unix_call_funcs[] =
     unix_gethostname,
     unix_getnameinfo,
 };
+
+C_ASSERT( ARRAYSIZE(__wine_unix_call_funcs) == ws_unix_funcs_count );
+
+#ifdef _WIN64
+
+typedef ULONG PTR32;
+
+struct WS_addrinfo32
+{
+    int   ai_flags;
+    int   ai_family;
+    int   ai_socktype;
+    int   ai_protocol;
+    PTR32 ai_addrlen;
+    PTR32 ai_canonname;
+    PTR32 ai_addr;
+    PTR32 ai_next;
+};
+
+struct WS_hostent32
+{
+    PTR32 h_name;
+    PTR32 h_aliases;
+    short h_addrtype;
+    short h_length;
+    PTR32 h_addr_list;
+};
+
+static NTSTATUS put_addrinfo32( const struct WS_addrinfo *info, struct WS_addrinfo32 *info32,
+                                unsigned int *size )
+{
+    struct WS_addrinfo32 *dst = info32, *prev = NULL;
+    const struct WS_addrinfo *src;
+    unsigned int needed_size = 0;
+
+    for (src = info; src != NULL; src = src->ai_next)
+    {
+        needed_size += sizeof(struct WS_addrinfo32);
+        if (src->ai_canonname) needed_size += strlen( src->ai_canonname ) + 1;
+        needed_size += src->ai_addrlen;
+    }
+
+    if (*size < needed_size)
+    {
+        *size = needed_size;
+        return ERROR_INSUFFICIENT_BUFFER;
+    }
+
+    memset( info32, 0, needed_size );
+
+    for (src = info; src != NULL; src = src->ai_next)
+    {
+        char *next = (char *)(dst + 1);
+
+        dst->ai_flags = src->ai_flags;
+        dst->ai_family = src->ai_family;
+        dst->ai_socktype = src->ai_socktype;
+        dst->ai_protocol = src->ai_protocol;
+        if (src->ai_canonname)
+        {
+            dst->ai_canonname = PtrToUlong( next );
+            strcpy( next, src->ai_canonname );
+            next += strlen(next) + 1;
+        }
+        dst->ai_addrlen = src->ai_addrlen;
+        dst->ai_addr = PtrToUlong(next);
+        memcpy( next, src->ai_addr, dst->ai_addrlen );
+        next += dst->ai_addrlen;
+        if (prev) prev->ai_next = PtrToUlong(dst);
+        prev = dst;
+        dst = (struct WS_addrinfo32 *)next;
+    }
+    return STATUS_SUCCESS;
+}
+
+static NTSTATUS put_hostent32( const struct WS_hostent *host, struct WS_hostent32 *host32,
+                               unsigned int *size )
+{
+    unsigned int needed_size = sizeof( struct WS_hostent32 ), alias_count = 0, addr_count = 0, i;
+    char *p;
+    ULONG *aliases, *addr_list;
+
+    needed_size += strlen( host->h_name ) + 1;
+
+    for (alias_count = 0; host->h_aliases[alias_count] != NULL; ++alias_count)
+        needed_size += sizeof(ULONG) + strlen( host->h_aliases[alias_count] ) + 1;
+    needed_size += sizeof(ULONG); /* null terminator */
+
+    for (addr_count = 0; host->h_addr_list[addr_count] != NULL; ++addr_count)
+        needed_size += sizeof(ULONG) + host->h_length;
+    needed_size += sizeof(ULONG); /* null terminator */
+
+    if (*size < needed_size)
+    {
+        *size = needed_size;
+        return ERROR_INSUFFICIENT_BUFFER;
+    }
+
+    memset( host32, 0, needed_size );
+
+    /* arrange the memory in the same order as windows >= XP */
+
+    host32->h_addrtype = host->h_addrtype;
+    host32->h_length = host->h_length;
+
+    aliases = (ULONG *)(host32 + 1);
+    addr_list = aliases + alias_count + 1;
+    p = (char *)(addr_list + addr_count + 1);
+
+    host32->h_aliases = PtrToUlong( aliases );
+    host32->h_addr_list = PtrToUlong( addr_list );
+
+    for (i = 0; i < addr_count; ++i)
+    {
+        addr_list[i] = PtrToUlong( p );
+        memcpy( p, host->h_addr_list[i], host->h_length );
+        p += host->h_length;
+    }
+
+    for (i = 0; i < alias_count; ++i)
+    {
+        size_t len = strlen( host->h_aliases[i] ) + 1;
+
+        aliases[i] = PtrToUlong( p );
+        memcpy( p, host->h_aliases[i], len );
+        p += len;
+    }
+
+    host32->h_name = PtrToUlong( p );
+    strcpy( p, host->h_name );
+    return STATUS_SUCCESS;
+}
+
+
+static NTSTATUS wow64_unix_getaddrinfo( void *args )
+{
+    struct
+    {
+        PTR32 node;
+        PTR32 service;
+        PTR32 hints;
+        PTR32 info;
+        PTR32 size;
+    } const *params32 = args;
+
+    NTSTATUS status;
+    struct WS_addrinfo hints;
+    struct getaddrinfo_params params =
+    {
+        ULongToPtr( params32->node ),
+        ULongToPtr( params32->service ),
+        NULL,
+        NULL,
+        ULongToPtr(params32->size)
+    };
+
+    if (params32->hints)
+    {
+        const struct WS_addrinfo32 *hints32 = ULongToPtr(params32->hints);
+        hints.ai_flags    = hints32->ai_flags;
+        hints.ai_family   = hints32->ai_family;
+        hints.ai_socktype = hints32->ai_socktype;
+        hints.ai_protocol = hints32->ai_protocol;
+        params.hints = &hints;
+    }
+
+    if (!(params.info = malloc( *params.size ))) return WSAENOBUFS;
+    status = unix_getaddrinfo( &params );
+    if (!status) put_addrinfo32( params.info, ULongToPtr(params32->info), ULongToPtr(params32->size) );
+    free( params.info );
+    return status;
+}
+
+
+static NTSTATUS wow64_unix_gethostbyaddr( void *args )
+{
+    struct
+    {
+        PTR32 addr;
+        int   len;
+        int   family;
+        PTR32 host;
+        PTR32 size;
+    } const *params32 = args;
+
+    NTSTATUS status;
+    struct gethostbyaddr_params params =
+    {
+        ULongToPtr( params32->addr ),
+        params32->len,
+        params32->family,
+        NULL,
+        ULongToPtr(params32->size)
+    };
+
+    if (!(params.host = malloc( *params.size ))) return WSAENOBUFS;
+    status = unix_gethostbyaddr( &params );
+    if (!status)
+        status = put_hostent32( params.host, ULongToPtr(params32->host), ULongToPtr(params32->size) );
+    free( params.host );
+    return status;
+}
+
+
+static NTSTATUS wow64_unix_gethostbyname( void *args )
+{
+    struct
+    {
+        PTR32 name;
+        PTR32 host;
+        PTR32 size;
+    } const *params32 = args;
+
+    NTSTATUS status;
+    struct gethostbyname_params params =
+    {
+        ULongToPtr( params32->name ),
+        NULL,
+        ULongToPtr(params32->size)
+    };
+
+    if (!(params.host = malloc( *params.size ))) return WSAENOBUFS;
+    status = unix_gethostbyname( &params );
+    if (!status)
+        status = put_hostent32( params.host, ULongToPtr(params32->host), ULongToPtr(params32->size) );
+    free( params.host );
+    return status;
+}
+
+
+static NTSTATUS wow64_unix_gethostname( void *args )
+{
+    struct
+    {
+        PTR32 name;
+        unsigned int size;
+    } const *params32 = args;
+
+    struct gethostname_params params = { ULongToPtr(params32->name), params32->size };
+
+    if (!unix_gethostname( &params )) return 0;
+    return errno_from_unix( errno );
+}
+
+
+static NTSTATUS wow64_unix_getnameinfo( void *args )
+{
+    struct
+    {
+        PTR32 addr;
+        int addr_len;
+        PTR32 host;
+        DWORD host_len;
+        PTR32 serv;
+        DWORD serv_len;
+        unsigned int flags;
+    } const *params32 = args;
+
+    struct getnameinfo_params params =
+    {
+        ULongToPtr( params32->addr ),
+        params32->addr_len,
+        ULongToPtr( params32->host ),
+        params32->host_len,
+        ULongToPtr( params32->serv ),
+        params32->serv_len,
+        params32->flags
+    };
+
+    return unix_getnameinfo( &params );
+}
+
+const unixlib_entry_t __wine_unix_call_wow64_funcs[] =
+{
+    wow64_unix_getaddrinfo,
+    wow64_unix_gethostbyaddr,
+    wow64_unix_gethostbyname,
+    wow64_unix_gethostname,
+    wow64_unix_getnameinfo,
+};
+
+C_ASSERT( ARRAYSIZE(__wine_unix_call_wow64_funcs) == ws_unix_funcs_count );
+
+#endif  /* _WIN64 */
