@@ -19,14 +19,13 @@
 #ifndef __WINE_WS2_32_PRIVATE_H
 #define __WINE_WS2_32_PRIVATE_H
 
+#include <assert.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #include "windef.h"
@@ -48,6 +47,7 @@
 #define USE_WC_PREFIX   /* For CMSG_DATA */
 #include "iphlpapi.h"
 #include "ip2string.h"
+#include "windns.h"
 #include "wine/afd.h"
 #include "wine/debug.h"
 #include "wine/unixlib.h"
@@ -59,6 +59,18 @@
       0, 0, { (DWORD_PTR)(__FILE__ ": " # cs) }}; \
     static CRITICAL_SECTION cs = { &cs##_debug, -1, 0, 0, 0, 0 }
 
+static inline char *strdupWtoA( const WCHAR *str )
+{
+    char *ret = NULL;
+    if (str)
+    {
+        DWORD len = WideCharToMultiByte( CP_ACP, 0, str, -1, NULL, 0, NULL, NULL );
+        if ((ret = malloc( len )))
+            WideCharToMultiByte( CP_ACP, 0, str, -1, ret, len, NULL, NULL );
+    }
+    return ret;
+}
+
 static const char magic_loopback_addr[] = {127, 12, 34, 56};
 
 const char *debugstr_sockaddr( const struct sockaddr *addr ) DECLSPEC_HIDDEN;
@@ -76,7 +88,7 @@ struct per_thread_data
     char ntoa_buffer[16]; /* 4*3 digits + 3 '.' + 1 '\0' */
 };
 
-extern int num_startup;
+extern int num_startup DECLSPEC_HIDDEN;
 
 struct per_thread_data *get_per_thread_data(void) DECLSPEC_HIDDEN;
 
@@ -129,8 +141,7 @@ enum ws_unix_funcs
     ws_unix_gethostbyname,
     ws_unix_gethostname,
     ws_unix_getnameinfo,
+    ws_unix_funcs_count,
 };
 
-extern unixlib_handle_t ws_unix_handle DECLSPEC_HIDDEN;
-
 #endif
