@@ -96,15 +96,14 @@ static WCHAR *get_wine_inf_path(void)
 
     if ((dir = _wgetenv( L"WINEBUILDDIR" )))
     {
-        if (!(name = HeapAlloc( GetProcessHeap(), 0,
-                                sizeof(L"\\loader\\wine.inf") + lstrlenW(dir) * sizeof(WCHAR) )))
+        if (!(name = malloc( sizeof(L"\\loader\\wine.inf") + wcslen(dir) * sizeof(WCHAR) )))
             return NULL;
         lstrcpyW( name, dir );
         lstrcatW( name, L"\\loader" );
     }
     else if ((dir = _wgetenv( L"WINEDATADIR" )))
     {
-        if (!(name = HeapAlloc( GetProcessHeap(), 0, sizeof(L"\\wine.inf") + lstrlenW(dir) * sizeof(WCHAR) )))
+        if (!(name = malloc( sizeof(L"\\wine.inf") + wcslen(dir) * sizeof(WCHAR) )))
             return NULL;
         lstrcpyW( name, dir );
     }
@@ -121,7 +120,7 @@ static BOOL update_timestamp( const WCHAR *config_dir, unsigned long timestamp )
     BOOL ret = FALSE;
     int fd, count;
     char buffer[100];
-    WCHAR *file = HeapAlloc( GetProcessHeap(), 0, lstrlenW(config_dir) * sizeof(WCHAR) + sizeof(L"\\.update-timestamp") );
+    WCHAR *file = malloc( wcslen(config_dir) * sizeof(WCHAR) + sizeof(L"\\.update-timestamp") );
 
     if (!file) return FALSE;
     lstrcpyW( file, config_dir );
@@ -154,7 +153,7 @@ static BOOL update_timestamp( const WCHAR *config_dir, unsigned long timestamp )
 
 done:
     if (fd != -1) close( fd );
-    HeapFree( GetProcessHeap(), 0, file );
+    free( file );
     return ret;
 }
 
@@ -257,11 +256,12 @@ static void create_user_shared_data(void)
     SYSTEM_BASIC_INFORMATION sbi;
     BOOLEAN *features;
     OBJECT_ATTRIBUTES attr = {sizeof(attr)};
-    UNICODE_STRING name;
+    UNICODE_STRING name = RTL_CONSTANT_STRING( L"\\KernelObjects\\__wine_user_shared_data" );
     NTSTATUS status;
     HANDLE handle;
+    ULONG i, machines[8];
+    HANDLE process = 0;
 
-    RtlInitUnicodeString( &name, L"\\KernelObjects\\__wine_user_shared_data" );
     InitializeObjectAttributes( &attr, &name, OBJ_OPENIF, NULL, NULL );
     if ((status = NtOpenSection( &handle, SECTION_ALL_ACCESS, &attr )))
     {
@@ -321,15 +321,46 @@ static void create_user_shared_data(void)
         features[PF_AVX_INSTRUCTIONS_AVAILABLE]           = !!(sci.ProcessorFeatureBits & CPU_FEATURE_AVX);
         features[PF_AVX2_INSTRUCTIONS_AVAILABLE]          = !!(sci.ProcessorFeatureBits & CPU_FEATURE_AVX2);
         break;
+
     case PROCESSOR_ARCHITECTURE_ARM:
         features[PF_ARM_VFP_32_REGISTERS_AVAILABLE]       = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_VFP_32);
         features[PF_ARM_NEON_INSTRUCTIONS_AVAILABLE]      = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_NEON);
         features[PF_ARM_V8_INSTRUCTIONS_AVAILABLE]        = (sci.ProcessorLevel >= 8);
         break;
+
     case PROCESSOR_ARCHITECTURE_ARM64:
         features[PF_ARM_V8_INSTRUCTIONS_AVAILABLE]        = TRUE;
         features[PF_ARM_V8_CRC32_INSTRUCTIONS_AVAILABLE]  = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V8_CRC32);
         features[PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V8_CRYPTO);
+        features[PF_COMPARE_EXCHANGE_DOUBLE]              = TRUE;
+        features[PF_NX_ENABLED]                           = TRUE;
+        features[PF_FASTFAIL_AVAILABLE]                   = TRUE;
+        /* add features for other architectures supported by wow64 */
+        if (!NtQuerySystemInformationEx( SystemSupportedProcessorArchitectures, &process, sizeof(process),
+                                         machines, sizeof(machines), NULL ))
+        {
+            for (i = 0; machines[i]; i++)
+            {
+                switch (LOWORD(machines[i]))
+                {
+                case IMAGE_FILE_MACHINE_ARMNT:
+                    features[PF_ARM_VFP_32_REGISTERS_AVAILABLE]  = TRUE;
+                    features[PF_ARM_NEON_INSTRUCTIONS_AVAILABLE] = TRUE;
+                    break;
+                case IMAGE_FILE_MACHINE_I386:
+                    features[PF_MMX_INSTRUCTIONS_AVAILABLE]    = TRUE;
+                    features[PF_XMMI_INSTRUCTIONS_AVAILABLE]   = TRUE;
+                    features[PF_RDTSC_INSTRUCTION_AVAILABLE]   = TRUE;
+                    features[PF_XMMI64_INSTRUCTIONS_AVAILABLE] = TRUE;
+                    features[PF_SSE3_INSTRUCTIONS_AVAILABLE]   = TRUE;
+                    features[PF_RDTSCP_INSTRUCTION_AVAILABLE]  = TRUE;
+                    features[PF_SSSE3_INSTRUCTIONS_AVAILABLE]  = TRUE;
+                    features[PF_SSE4_1_INSTRUCTIONS_AVAILABLE] = TRUE;
+                    features[PF_SSE4_2_INSTRUCTIONS_AVAILABLE] = TRUE;
+                    break;
+                }
+            }
+        }
         break;
     }
     data->ActiveProcessorCount = NtCurrentTeb()->Peb->NumberOfProcessors;
@@ -387,29 +418,10 @@ static void get_vendorid( WCHAR *buf )
     regs_to_str( regs + 1, 12, buf );
 }
 
-static void get_namestring( WCHAR *buf )
-{
-    int regs[4] = {0, 0, 0, 0};
-    int i;
-
-    __cpuid( regs, 0x80000000 );
-    if (regs[0] >= 0x80000004)
-    {
-        __cpuid( regs, 0x80000002 );
-        regs_to_str( regs, 16, buf );
-        __cpuid( regs, 0x80000003 );
-        regs_to_str( regs, 16, buf + 16 );
-        __cpuid( regs, 0x80000004 );
-        regs_to_str( regs, 16, buf + 32 );
-    }
-    for (i = lstrlenW(buf) - 1; i >= 0 && buf[i] == ' '; i--) buf[i] = 0;
-}
-
 #else  /* __i386__ || __x86_64__ */
 
 static void get_identifier( WCHAR *buf, size_t size, const WCHAR *arch ) { }
 static void get_vendorid( WCHAR *buf ) { }
-static void get_namestring( WCHAR *buf ) { }
 
 #endif  /* __i386__ || __x86_64__ */
 
@@ -526,7 +538,7 @@ static inline WCHAR *heap_strdupAW( const char *src )
     WCHAR *dst;
     if (!src) return NULL;
     len = MultiByteToWideChar( CP_ACP, 0, src, -1, NULL, 0 );
-    if ((dst = HeapAlloc( GetProcessHeap(), 0, len * sizeof(*dst) ))) MultiByteToWideChar( CP_ACP, 0, src, -1, dst, len );
+    if ((dst = malloc( len * sizeof(*dst) ))) MultiByteToWideChar( CP_ACP, 0, src, -1, dst, len );
     return dst;
 }
 
@@ -549,7 +561,7 @@ static void set_value_from_smbios_string( HKEY key, const WCHAR *value, BYTE id,
     WCHAR *str;
     str = get_smbios_string( id, buf, offset, buflen );
     set_reg_value( key, value, str ? str : L"" );
-    HeapFree( GetProcessHeap(), 0, str );
+    free( str );
 }
 
 static void create_bios_baseboard_values( HKEY bios_key, const char *buf, UINT len )
@@ -634,7 +646,7 @@ static void create_bios_key( HKEY system_key )
         return;
 
     len = GetSystemFirmwareTable( RSMB, 0, NULL, 0 );
-    if (!(buf = HeapAlloc( GetProcessHeap(), 0, len ))) goto done;
+    if (!(buf = malloc( len ))) goto done;
     len = GetSystemFirmwareTable( RSMB, 0, buf, len );
 
     create_bios_baseboard_values( bios_key, buf, len );
@@ -642,7 +654,7 @@ static void create_bios_key( HKEY system_key )
     create_bios_system_values( bios_key, buf, len );
 
 done:
-    HeapFree( GetProcessHeap(), 0, buf );
+    free( buf );
     RegCloseKey( bios_key );
 }
 
@@ -654,13 +666,15 @@ static void create_hardware_registry_keys(void)
     SYSTEM_CPU_INFORMATION sci;
     PROCESSOR_POWER_INFORMATION* power_info;
     ULONG sizeof_power_info = sizeof(PROCESSOR_POWER_INFORMATION) * NtCurrentTeb()->Peb->NumberOfProcessors;
-    WCHAR id[60], namestr[49], vendorid[13];
+    ULONG name_buffer[16];
+    WCHAR id[60], vendorid[13];
 
-    get_namestring( namestr );
     get_vendorid( vendorid );
     NtQuerySystemInformation( SystemCpuInformation, &sci, sizeof(sci), NULL );
+    if (NtQuerySystemInformation( SystemProcessorBrandString, name_buffer, sizeof(name_buffer), NULL ))
+        name_buffer[0] = 0;
 
-    power_info = HeapAlloc( GetProcessHeap(), 0, sizeof_power_info );
+    power_info = malloc( sizeof_power_info );
     if (power_info == NULL)
         return;
     if (NtPowerInformation( ProcessorInformation, NULL, 0, power_info, sizeof_power_info ))
@@ -687,7 +701,7 @@ static void create_hardware_registry_keys(void)
     if (RegCreateKeyExW( HKEY_LOCAL_MACHINE, L"Hardware\\Description\\System", 0, NULL,
                          REG_OPTION_VOLATILE, KEY_ALL_ACCESS, NULL, &system_key, NULL ))
     {
-        HeapFree( GetProcessHeap(), 0, power_info );
+        free( power_info );
         return;
     }
 
@@ -725,7 +739,8 @@ static void create_hardware_registry_keys(void)
             RegSetValueExW( hkey, L"FeatureSet", 0, REG_DWORD, (BYTE *)&sci.ProcessorFeatureBits, sizeof(DWORD) );
             set_reg_value( hkey, L"Identifier", id );
             /* TODO: report ARM properly */
-            set_reg_value( hkey, L"ProcessorNameString", namestr );
+            RegSetValueExA( hkey, "ProcessorNameString", 0, REG_SZ, (const BYTE *)name_buffer,
+                            strlen( (char *)name_buffer ) + 1 );
             set_reg_value( hkey, L"VendorIdentifier", vendorid );
             RegSetValueExW( hkey, L"~MHz", 0, REG_DWORD, (BYTE *)&power_info[i].MaxMhz, sizeof(DWORD) );
             RegCloseKey( hkey );
@@ -745,7 +760,7 @@ static void create_hardware_registry_keys(void)
     RegCloseKey( fpu_key );
     RegCloseKey( cpu_key );
     RegCloseKey( system_key );
-    HeapFree( GetProcessHeap(), 0, power_info );
+    free( power_info );
 }
 
 
@@ -918,9 +933,9 @@ static BOOL wininit(void)
     {
         if (!(res = GetPrivateProfileSectionW( L"rename", buffer, size, L"wininit.ini" ))) return TRUE;
         if (res < size - 2) break;
-        if (buffer != initial_buffer) HeapFree( GetProcessHeap(), 0, buffer );
+        if (buffer != initial_buffer) free( buffer );
         size *= 2;
-        if (!(buffer = HeapAlloc( GetProcessHeap(), 0, size * sizeof(WCHAR) ))) return FALSE;
+        if (!(buffer = malloc( size * sizeof(WCHAR) ))) return FALSE;
     }
 
     for (str = buffer; *str; str += lstrlenW(str) + 1)
@@ -949,7 +964,7 @@ static BOOL wininit(void)
         str = value;
     }
 
-    if (buffer != initial_buffer) HeapFree( GetProcessHeap(), 0, buffer );
+    if (buffer != initial_buffer) free( buffer );
 
     if( !MoveFileExW( L"wininit.ini", L"wininit.bak", MOVEFILE_REPLACE_EXISTING) )
     {
@@ -974,7 +989,7 @@ static void pendingRename(void)
 
     if (RegQueryValueExW( hSession, L"PendingFileRenameOperations", NULL, NULL, NULL, &dataLength ))
         goto end;
-    if (!(buffer = HeapAlloc( GetProcessHeap(), 0, dataLength ))) goto end;
+    if (!(buffer = malloc( dataLength ))) goto end;
 
     if (RegQueryValueExW( hSession, L"PendingFileRenameOperations", NULL, NULL,
                           (LPBYTE)buffer, &dataLength ))
@@ -1022,7 +1037,7 @@ static void pendingRename(void)
     RegDeleteValueW(hSession, L"PendingFileRenameOperations");
 
 end:
-    HeapFree(GetProcessHeap(), 0, buffer);
+    free( buffer );
     RegCloseKey( hSession );
 }
 
@@ -1096,12 +1111,12 @@ static void process_run_key( HKEY key, const WCHAR *keyname, BOOL delete, BOOL s
         WINE_TRACE( "No commands to execute.\n" );
         goto end;
     }
-    if (!(cmdline = HeapAlloc( GetProcessHeap(), 0, max_cmdline )))
+    if (!(cmdline = malloc( max_cmdline )))
     {
         WINE_ERR( "Couldn't allocate memory for the commands to be executed.\n" );
         goto end;
     }
-    if (!(value = HeapAlloc( GetProcessHeap(), 0, ++max_value * sizeof(*value) )))
+    if (!(value = malloc( ++max_value * sizeof(*value) )))
     {
         WINE_ERR( "Couldn't allocate memory for the value names.\n" );
         goto end;
@@ -1133,8 +1148,8 @@ static void process_run_key( HKEY key, const WCHAR *keyname, BOOL delete, BOOL s
     }
 
 end:
-    HeapFree( GetProcessHeap(), 0, value );
-    HeapFree( GetProcessHeap(), 0, cmdline );
+    free( value );
+    free( cmdline );
     RegCloseKey( runkey );
     WINE_TRACE( "Done.\n" );
 }
@@ -1204,7 +1219,7 @@ static int ProcessWindowsFileProtection(void)
         if (!RegQueryValueExW( hkey, L"SFCDllCacheDir", 0, NULL, NULL, &sz))
         {
             sz += sizeof(WCHAR);
-            dllcache = HeapAlloc(GetProcessHeap(),0,sz + sizeof(L"\\*"));
+            dllcache = malloc( sz + sizeof(L"\\*") );
             RegQueryValueExW( hkey, L"SFCDllCacheDir", 0, NULL, (LPBYTE)dllcache, &sz);
             lstrcatW( dllcache, L"\\*" );
         }
@@ -1214,7 +1229,7 @@ static int ProcessWindowsFileProtection(void)
     if (!dllcache)
     {
         DWORD sz = GetSystemDirectoryW( NULL, 0 );
-        dllcache = HeapAlloc( GetProcessHeap(), 0, sz * sizeof(WCHAR) + sizeof(L"\\dllcache\\*"));
+        dllcache = malloc( sz * sizeof(WCHAR) + sizeof(L"\\dllcache\\*") );
         GetSystemDirectoryW( dllcache, sz );
         lstrcatW( dllcache, L"\\dllcache\\*" );
     }
@@ -1260,7 +1275,7 @@ static int ProcessWindowsFileProtection(void)
         find_rc = FindNextFileW(find_handle,&finddata);
     }
     FindClose(find_handle);
-    HeapFree(GetProcessHeap(),0,dllcache);
+    free(dllcache);
     return 1;
 }
 
@@ -1311,10 +1326,10 @@ static INT_PTR CALLBACK wait_dlgproc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp
             SendDlgItemMessageW( hwnd, IDC_WAITICON, STM_SETICON, (WPARAM)icon, 0 );
             SendDlgItemMessageW( hwnd, IDC_WAITTEXT, WM_GETTEXT, 1024, (LPARAM)text );
             len = lstrlenW(text) + lstrlenW(name) + 1;
-            buffer = HeapAlloc( GetProcessHeap(), 0, len * sizeof(WCHAR) );
+            buffer = malloc( len * sizeof(WCHAR) );
             swprintf( buffer, len, text, name );
             SendDlgItemMessageW( hwnd, IDC_WAITTEXT, WM_SETTEXT, 0, (LPARAM)buffer );
-            HeapFree( GetProcessHeap(), 0, buffer );
+            free( buffer );
         }
         break;
     }
@@ -1346,7 +1361,7 @@ static HANDLE start_rundll32( const WCHAR *inf_path, const WCHAR *install, WORD
 
     len = lstrlenW(app) + ARRAY_SIZE(L" setupapi,InstallHinfSection DefaultInstall 128 ") + lstrlenW(inf_path);
 
-    if (!(buffer = HeapAlloc( GetProcessHeap(), 0, len * sizeof(WCHAR) ))) return 0;
+    if (!(buffer = malloc( len * sizeof(WCHAR) ))) return 0;
     swprintf( buffer, len, L"%s setupapi,InstallHinfSection %s 128 %s", app, install, inf_path );
 
     if (CreateProcessW( app, buffer, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi ))
@@ -1354,7 +1369,7 @@ static HANDLE start_rundll32( const WCHAR *inf_path, const WCHAR *install, WORD
     else
         pi.hProcess = 0;
 
-    HeapFree( GetProcessHeap(), 0, buffer );
+    free( buffer );
     return pi.hProcess;
 }
 
@@ -1482,20 +1497,23 @@ static void update_wineprefix( BOOL force )
             HWND hwnd = show_wait_window();
             for (;;)
             {
-                MSG msg;
-                DWORD res = MsgWaitForMultipleObjects( 1, &process, FALSE, INFINITE, QS_ALLINPUT );
-                if (res == WAIT_OBJECT_0)
+                if (process)
                 {
+                    MSG msg;
+                    DWORD res = MsgWaitForMultipleObjects( 1, &process, FALSE, INFINITE, QS_ALLINPUT );
+                    if (res != WAIT_OBJECT_0)
+                    {
+                        while (PeekMessageW( &msg, 0, 0, 0, PM_REMOVE )) DispatchMessageW( &msg );
+                        continue;
+                    }
                     CloseHandle( process );
-                    if (!machines[count]) break;
-                    if (HIWORD(machines[count]) & 4 /* native machine */)
-                        process = start_rundll32( inf_path, L"DefaultInstall", IMAGE_FILE_MACHINE_TARGET_HOST );
-                    else
-                        process = start_rundll32( inf_path, L"Wow64Install", LOWORD(machines[count]) );
-                    count++;
-                    if (!process) break;
                 }
-                else while (PeekMessageW( &msg, 0, 0, 0, PM_REMOVE )) DispatchMessageW( &msg );
+                if (!machines[count]) break;
+                if (HIWORD(machines[count]) & 4 /* native machine */)
+                    process = start_rundll32( inf_path, L"DefaultInstall", IMAGE_FILE_MACHINE_TARGET_HOST );
+                else
+                    process = start_rundll32( inf_path, L"Wow64Install", LOWORD(machines[count]) );
+                count++;
             }
             DestroyWindow( hwnd );
         }
@@ -1506,7 +1524,7 @@ static void update_wineprefix( BOOL force )
     }
 
 done:
-    HeapFree( GetProcessHeap(), 0, inf_path );
+    free( inf_path );
 }
 
 /* Process items in the StartUp group of the user's Programs under the Start Menu. Some installers put
@@ -1609,7 +1627,7 @@ int __cdecl main( int argc, char *argv[] )
     BOOL end_session, force, init, kill, restart, shutdown, update;
     HANDLE event;
     OBJECT_ATTRIBUTES attr;
-    UNICODE_STRING nameW;
+    UNICODE_STRING nameW = RTL_CONSTANT_STRING( L"\\KernelObjects\\__wineboot_event" );
     BOOL is_wow64;
 
     end_session = force = init = kill = restart = shutdown = update = FALSE;
@@ -1690,7 +1708,6 @@ int __cdecl main( int argc, char *argv[] )
 
     /* create event to be inherited by services.exe */
     InitializeObjectAttributes( &attr, &nameW, OBJ_OPENIF | OBJ_INHERIT, 0, NULL );
-    RtlInitUnicodeString( &nameW, L"\\KernelObjects\\__wineboot_event" );
     NtCreateEvent( &event, EVENT_ALL_ACCESS, &attr, NotificationEvent, 0 );
 
     ResetEvent( event );  /* in case this is a restart */
