@@ -222,7 +222,7 @@ static void output_relay_debug( DLLSPEC *spec )
     /* first the table of entry point offsets */
 
     output( "\t%s\n", get_asm_rodata_section() );
-    output( "\t.align %d\n", get_alignment(4) );
+    output( "\t.balign 4\n" );
     output( ".L__wine_spec_relay_entry_point_offsets:\n" );
 
     for (i = spec->base; i <= spec->limit; i++)
@@ -256,7 +256,7 @@ static void output_relay_debug( DLLSPEC *spec )
         switch (target.cpu)
         {
         case CPU_i386:
-            output( "\t.align %d\n", get_alignment(4) );
+            output( "\t.balign 4\n" );
             output( "\t.long 0x90909090,0x90909090\n" );
             output( "__wine_spec_relay_entry_point_%d:\n", i );
             output_cfi( ".cfi_startproc" );
@@ -298,9 +298,8 @@ static void output_relay_debug( DLLSPEC *spec )
                 for (j = 0; j < odp->u.func.nb_args && !has_float; j++)
                     has_float = is_float_arg( odp, j );
 
-            output( "\t.align %d\n", get_alignment(4) );
+            output( "\t.balign 4\n" );
             output( "__wine_spec_relay_entry_point_%d:\n", i );
-            output_cfi( ".cfi_startproc" );
             output( "\tpush {r0-r3}\n" );
             output( "\tmov r2, SP\n");
             if (has_float) output( "\tvpush {s0-s15}\n" );
@@ -324,33 +323,34 @@ static void output_relay_debug( DLLSPEC *spec )
             output( "\tadd SP, #%u\n", 24 + (has_float ? 64 : 0) );
             output( "\tbx IP\n");
             if (UsePIC) output( "2:\t.long .L__wine_spec_relay_descr-1b-%u\n", thumb_mode ? 4 : 8 );
-            output_cfi( ".cfi_endproc" );
             break;
         }
 
         case CPU_ARM64:
-            output( "\t.align %d\n", get_alignment(4) );
+        {
+            int stack_size = 16 * ((min(odp->u.func.nb_args, 8) + 1) / 2);
+
+            output( "\t.balign 4\n" );
             output( "__wine_spec_relay_entry_point_%d:\n", i );
-            output_cfi( ".cfi_startproc" );
-            switch (odp->u.func.nb_args)
+            output_seh( ".seh_proc __wine_spec_relay_entry_point_%d", i );
+            output( "\tstp x29, x30, [sp, #-%u]!\n", stack_size + 16 );
+            output_seh( ".seh_save_fplr_x %u", stack_size + 16 );
+            output( "\tmov x29, sp\n" );
+            output_seh( ".seh_set_fp" );
+            output_seh( ".seh_endprologue" );
+            switch (stack_size)
             {
-            default:
-            case 8:
-            case 7:  output( "\tstp x6, x7, [SP,#-16]!\n" );
+            case 64: output( "\tstp x6, x7, [sp, #64]\n" );
             /* fall through */
-            case 6:
-            case 5:  output( "\tstp x4, x5, [SP,#-16]!\n" );
+            case 48: output( "\tstp x4, x5, [sp, #48]\n" );
             /* fall through */
-            case 4:
-            case 3:  output( "\tstp x2, x3, [SP,#-16]!\n" );
+            case 32: output( "\tstp x2, x3, [sp, #32]\n" );
             /* fall through */
-            case 2:
-            case 1:  output( "\tstp x0, x1, [SP,#-16]!\n" );
+            case 16: output( "\tstp x0, x1, [sp, #16]\n" );
             /* fall through */
-            case 0:  break;
+            default: break;
             }
-            output( "\tmov x2, SP\n");
-            output( "\tstp x29, x30, [SP,#-16]!\n" );
+            output( "\tadd x2, sp, #16\n");
             output( "\tstp x8, x9, [SP,#-16]!\n" );
             output( "\tmov w1, #%u\n", odp->u.func.args_str_offset << 16 );
             if (i - spec->base) output( "\tadd w1, w1, #%u\n", i - spec->base );
@@ -358,19 +358,19 @@ static void output_relay_debug( DLLSPEC *spec )
             output( "\tadd x0, x0, #%s\n", arm64_pageoff(".L__wine_spec_relay_descr") );
             output( "\tldr x3, [x0, #8]\n");
             output( "\tblr x3\n");
-            output( "\tadd SP, SP, #16\n" );
-            output( "\tldp x29, x30, [SP], #16\n" );
-            if (odp->u.func.nb_args)
-                output( "\tadd SP, SP, #%u\n", 8 * ((min(odp->u.func.nb_args, 8) + 1) & ~1) );
+            output( "\tmov sp, x29\n" );
+            output( "\tldp x29, x30, [sp], #%u\n", stack_size + 16 );
             output( "\tret\n");
-            output_cfi( ".cfi_endproc" );
+            output_seh( ".seh_endproc" );
             break;
+        }
 
         case CPU_x86_64:
-            output( "\t.align %d\n", get_alignment(4) );
+            output( "\t.balign 4\n" );
             output( "\t.long 0x90909090,0x90909090\n" );
             output( "__wine_spec_relay_entry_point_%d:\n", i );
-            output_cfi( ".cfi_startproc" );
+            output_seh( ".seh_proc __wine_spec_relay_entry_point_%d", i );
+            output_seh( ".seh_endprologue" );
             switch (odp->u.func.nb_args)
             {
             default: output( "\tmovq %%%s,32(%%rsp)\n", is_float_arg( odp, 3 ) ? "xmm3" : "r9" );
@@ -387,7 +387,7 @@ static void output_relay_debug( DLLSPEC *spec )
             output( "\tleaq .L__wine_spec_relay_descr(%%rip),%%rcx\n" );
             output( "\tcallq *8(%%rcx)\n" );
             output( "\tret\n" );
-            output_cfi( ".cfi_endproc" );
+            output_seh( ".seh_endproc" );
             break;
 
         default:
@@ -414,7 +414,7 @@ void output_exports( DLLSPEC *spec )
 
     output( "\n/* export table */\n\n" );
     output( "\t%s\n", get_asm_export_section() );
-    output( "\t.align %d\n", get_alignment(4) );
+    output( "\t.balign 4\n" );
     output( ".L__wine_spec_exports:\n" );
 
     /* export directory header */
@@ -541,11 +541,11 @@ void output_exports( DLLSPEC *spec )
         if (is_pe())
         {
             output( "\t.data\n" );
-            output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+            output( "\t.balign %u\n", get_ptr_size() );
         }
         else
         {
-            output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+            output( "\t.balign %u\n", get_ptr_size() );
             output( ".L__wine_spec_exports_end:\n" );
         }
 
@@ -561,7 +561,7 @@ void output_exports( DLLSPEC *spec )
     }
     else if (!is_pe())
     {
-            output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+            output( "\t.balign %u\n", get_ptr_size() );
             output( ".L__wine_spec_exports_end:\n" );
             output( "\t%s 0\n", get_asm_ptr_keyword() );
     }
@@ -578,11 +578,10 @@ void output_exports( DLLSPEC *spec )
 
         name = odp->name ? odp->name : odp->export_name;
 
-        output( "\t.align %d\n", get_alignment(4) );
+        output( "\t.balign 4\n" );
         output( "\t.long 0x90909090,0x90909090\n" );
         if (name) output( "%s_%s:\n", asm_name("__wine_spec_imp"), name );
         else output( "%s_%u:\n", asm_name("__wine_spec_imp"), i );
-        output_cfi( ".cfi_startproc" );
 
         switch (target.cpu)
         {
@@ -603,7 +602,6 @@ void output_exports( DLLSPEC *spec )
         default:
             assert(0);
         }
-        output_cfi( ".cfi_endproc" );
     }
 }
 
@@ -629,7 +627,7 @@ void output_module( DLLSPEC *spec )
         return;  /* nothing to do */
     case PLATFORM_APPLE:
         output( "\t.text\n" );
-        output( "\t.align %d\n", get_alignment(page_size) );
+        output( "\t.balign %u\n", page_size );
         output( "__wine_spec_pe_header:\n" );
         output( "\t.space 65536\n" );
         break;
@@ -664,7 +662,7 @@ void output_module( DLLSPEC *spec )
     /* Output the NT header */
 
     output( "\n\t.data\n" );
-    output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+    output( "\t.balign %u\n", get_ptr_size() );
     output( "\t.globl %s\n", asm_name("__wine_spec_nt_header") );
     output( "%s:\n", asm_name("__wine_spec_nt_header") );
     output( ".L__wine_spec_rva_base:\n" );
@@ -735,6 +733,8 @@ void output_module( DLLSPEC *spec )
         data_dirs[1] = ".L__wine_spec_imports";   /* DataDirectory[IMAGE_DIRECTORY_ENTRY_IMPORT] */
     if (spec->nb_resources)
         data_dirs[2] = ".L__wine_spec_resources"; /* DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE] */
+    if (has_delay_imports())
+        data_dirs[13] = ".L__wine_spec_delay_imports"; /* DataDirectory[IMAGE_DIRECTORY_ENTRY_DELAY_IMPORT] */
 
     output_data_directories( data_dirs );
 
@@ -1111,8 +1111,16 @@ static void output_pe_file( DLLSPEC *spec, const char signature[32] )
     put_dword( 0 );                                  /* SizeOfUninitializedData */
     put_dword( code_size ? pe.sec[0].rva : 0 );      /* AddressOfEntryPoint */
     put_dword( code_size ? pe.sec[0].rva : 0 );      /* BaseOfCode */
-    if (get_ptr_size() == 4) put_dword( 0 );         /* BaseOfData */
-    put_pword( 0x10000000 );                         /* ImageBase */
+    if (get_ptr_size() == 4)
+    {
+        put_dword( 0 );                              /* BaseOfData */
+        put_dword( 0x10000000 );                     /* ImageBase */
+    }
+    else
+    {
+        put_dword( 0x80000000 );                     /* ImageBase */
+        put_dword( 0x00000001 );
+    }
     put_dword( pe.section_align );                   /* SectionAlignment */
     put_dword( pe.file_align );                      /* FileAlignment */
     put_word( 1 );                                   /* MajorOperatingSystemVersion */
@@ -1446,9 +1454,9 @@ static void fixup_elf32( const char *name, int fd, void *header, size_t header_s
     {
         switch (dyn[i].d_tag)
         {
-        case 25: dyn[i].d_tag = 0x60009990; break;  /* DT_INIT_ARRAY */
-        case 27: dyn[i].d_tag = 0x60009991; break;  /* DT_INIT_ARRAYSZ */
-        case 12: dyn[i].d_tag = 0x60009992; break;  /* DT_INIT */
+        case 25: dyn[i].d_tag = 0x60009994; break;  /* DT_INIT_ARRAY */
+        case 27: dyn[i].d_tag = 0x60009995; break;  /* DT_INIT_ARRAYSZ */
+        case 12: dyn[i].d_tag = 0x60009996; break;  /* DT_INIT */
         }
     }
     lseek( fd, phdr->p_offset, SEEK_SET );
@@ -1515,9 +1523,9 @@ static void fixup_elf64( const char *name, int fd, void *header, size_t header_s
     {
         switch (dyn[i].d_tag)
         {
-        case 25: dyn[i].d_tag = 0x60009990; break;  /* DT_INIT_ARRAY */
-        case 27: dyn[i].d_tag = 0x60009991; break;  /* DT_INIT_ARRAYSZ */
-        case 12: dyn[i].d_tag = 0x60009992; break;  /* DT_INIT */
+        case 25: dyn[i].d_tag = 0x60009994; break;  /* DT_INIT_ARRAY */
+        case 27: dyn[i].d_tag = 0x60009995; break;  /* DT_INIT_ARRAYSZ */
+        case 12: dyn[i].d_tag = 0x60009996; break;  /* DT_INIT */
         }
     }
     lseek( fd, phdr->p_offset, SEEK_SET );
